from _typeshed import Incomplete
from collections.abc import Callable
from typing import Literal

def register_reset(func: Callable[[], Callable[[], object] | None]) -> None: ...
def _reset() -> None: ...

allowTableBoundsErrors: int
shapeChecking: int
defaultEncoding: str
defaultGraphicsFontName: str
pageCompression: int
useA85: int
defaultPageSize: tuple[float, float]
defaultImageCaching: int
warnOnMissingFontGlyphs: int
verbose: int
showBoundary: int
emptyTableAction: str
invariant: int
eps_preview_transparent: Incomplete
eps_preview: int
eps_ttf_embed: int
eps_ttf_embed_uid: int
overlapAttachedSpace: int
longTableOptimize: int
autoConvertEncoding: int
_FUZZ: float
wrapA85: int
fsEncodings: tuple[Literal["utf8"], Literal["cp1252"], Literal["cp430"]]
odbc_driver: str
platypus_link_underline: int
canvas_basefontname: str
allowShortTableRows: int
imageReaderFlags: int
paraFontSizeHeightOffset: int
canvas_baseColor: Incomplete
ignoreContainerActions: int
ttfAsciiReadable: int
pdfMultiLine: int
pdfComments: int
debug: int
listWrapOnFakeWidth: int
underlineWidth: str
underlineOffset: str
underlineGap: str
strikeWidth: str
strikeOffset: str
strikeGap: str
decimalSymbol: str
errorOnDuplicatePageLabelPage: int
autoGenerateMissingTTFName: int
allowTTFSubsetting: list[str]
spaceShrinkage: float
hyphenationLang: str
uriWasteReduce: int
embeddedHyphenation: int
hyphenationMinWordLength: int
reserveTTFNotdef: int
documentLang: Incomplete
encryptionStrength: int
trustedHosts: list[str] | None
trustedSchemes: list[str]
renderPMBackend: str
xmlParser: str
textPaths: str
toColorCanUse: str
defCWRF: float
unShapedFontGlob: list[str] | None
T1SearchPath: list[str]
TTFSearchPath: list[str]
CMapSearchPath: list[str]



from .errors import (
    InvalidTokenError as InvalidTokenError,
    MismatchingStateError as MismatchingStateError,
    MissingRequestTokenError as MissingRequestTokenError,
    MissingTokenError as MissingTokenError,
    OAuthError as OAuthError,
    TokenExpiredError as TokenExpiredError,
    UnsupportedTokenTypeError as UnsupportedTokenTypeError,
)
from .framework_integration import FrameworkIntegration as FrameworkIntegration
from .registry import BaseOAuth as BaseOAuth
from .sync_app import BaseApp as BaseApp, OAuth1Mixin as OAuth1Mixin, OAuth2Mixin as OAuth2Mixin
from .sync_openid import OpenIDMixin as OpenIDMixin

__all__ = [
    "BaseOAuth",
    "BaseApp",
    "OAuth1Mixin",
    "OAuth2Mixin",
    "OpenIDMixin",
    "FrameworkIntegration",
    "OAuthError",
    "MissingRequestTokenError",
    "MissingTokenError",
    "TokenExpiredError",
    "InvalidTokenError",
    "UnsupportedTokenTypeError",
    "MismatchingStateError",
]



from _typeshed import StrOrBytesPath, StrPath
from collections.abc import Iterable

class RecentFiles:
    RecentFiles: list[RecentFile]
    filename: str
    def __init__(self) -> None: ...
    def parse(self, filename: StrPath | None = None) -> None: ...
    def write(self, filename: StrOrBytesPath | None = None) -> None: ...
    def getFiles(
        self, mimetypes: Iterable[str] | None = None, groups: Iterable[str] | None = None, limit: int = 0
    ) -> list[RecentFile]: ...
    def addFile(self, item: StrPath, mimetype: str, groups: Iterable[str] | None = None, private: bool = False) -> None: ...
    def deleteFile(self, item: RecentFile | StrPath) -> None: ...
    def sort(self) -> None: ...

class RecentFile:
    URI: str
    MimeType: str
    Timestamp: str
    Private: bool
    Groups: list[str]
    def __init__(self) -> None: ...
    def __cmp__(self, other: RecentFile) -> int: ...
    def __lt__(self, other: RecentFile) -> bool: ...
    def __eq__(self, other: object) -> bool: ...



from .clip import clip as clip
from .geocoding import geocode as geocode, reverse_geocode as reverse_geocode
from .overlay import overlay as overlay
from .sjoin import sjoin as sjoin, sjoin_nearest as sjoin_nearest
from .util import collect as collect

__all__ = ["collect", "geocode", "overlay", "reverse_geocode", "sjoin", "sjoin_nearest", "clip"]



from _typeshed import Incomplete

from aws_xray_sdk.core.context import Context

from .sampling_rule import SamplingRule

class ServiceConnector:
    def __init__(self) -> None: ...
    def fetch_sampling_rules(self) -> list[SamplingRule]: ...
    def fetch_sampling_target(self, rules) -> tuple[Incomplete, int]: ...
    def setup_xray_client(self, ip: str, port: str | int, client) -> None: ...
    @property
    def context(self) -> Context: ...
    @context.setter
    def context(self, v: Context) -> None: ...



from _typeshed import Incomplete
from collections.abc import Mapping, Sequence
from typing import Any
from typing_extensions import Self

from jwcrypto import common
from jwcrypto.common import JWException, JWSEHeaderParameter, JWSEHeaderRegistry
from jwcrypto.jwk import JWK, JWKSet

default_max_compressed_size: int
JWEHeaderRegistry: Mapping[str, JWSEHeaderParameter]
default_allowed_algs: Sequence[str]

class InvalidJWEData(JWException):
    def __init__(self, message: str | None = None, exception: BaseException | None = None) -> None: ...

InvalidCEKeyLength = common.InvalidCEKeyLength
InvalidJWEKeyLength = common.InvalidJWEKeyLength
InvalidJWEKeyType = common.InvalidJWEKeyType
InvalidJWEOperation = common.InvalidJWEOperation

class JWE:
    objects: dict[str, Any]
    plaintext: bytes | None
    header_registry: JWSEHeaderRegistry
    cek: Incomplete
    decryptlog: list[str] | None
    def __init__(
        self,
        plaintext: str | bytes | None = None,
        protected: str | None = None,
        unprotected: str | None = None,
        aad: bytes | None = None,
        algs: list[str] | None = None,
        recipient: str | None = None,
        header: str | None = None,
        header_registry: Mapping[str, JWSEHeaderParameter] | None = None,
    ) -> None: ...
    @property
    def allowed_algs(self) -> list[str]: ...
    @allowed_algs.setter
    def allowed_algs(self, algs: list[str]) -> None: ...
    def add_recipient(self, key: JWK, header: dict[str, Any] | str | None = None) -> None: ...
    def serialize(self, compact: bool = False) -> str: ...
    def decrypt(self, key: JWK | JWKSet) -> None: ...
    def deserialize(self, raw_jwe: str | bytes, key: JWK | JWKSet | None = None) -> None: ...
    @property
    def payload(self) -> bytes: ...
    @property
    def jose_header(self) -> dict[Incomplete, Incomplete]: ...
    @classmethod
    def from_jose_token(cls, token: str | bytes) -> Self: ...
    def __eq__(self, other: object) -> bool: ...



import datetime
import sys
import winreg
from _typeshed import Incomplete
from typing import Any, SupportsIndex

if sys.platform == "win32":
    handle: winreg.HKEYType
    tzparent: winreg.HKEYType
    parentsize: int
    localkey: winreg.HKEYType
    WEEKS: datetime.timedelta
    def list_timezones() -> list[str]: ...

    class win32tz(datetime.tzinfo):
        data: win32tz_data
        def __init__(self, name: str | None) -> None: ...
        def utcoffset(self, dt: datetime.datetime) -> datetime.timedelta: ...  # type: ignore[override]
        def dst(self, dt: datetime.datetime) -> datetime.timedelta: ...  # type: ignore[override]
        def tzname(self, dt: datetime.datetime) -> str | None: ...  # type: ignore[override]

    def pickNthWeekday(
        year: SupportsIndex, month: SupportsIndex, dayofweek: int, hour: SupportsIndex, minute: SupportsIndex, whichweek: int
    ) -> datetime.datetime | None: ...

    class win32tz_data:
        display: Incomplete
        dstname: Incomplete
        stdname: Incomplete
        stdoffset: Incomplete
        dstoffset: Incomplete
        stdmonth: Incomplete
        stddayofweek: Incomplete
        stdweeknumber: Incomplete
        stdhour: Incomplete
        stdminute: Incomplete
        dstmonth: Incomplete
        dstdayofweek: Incomplete
        dstweeknumber: Incomplete
        dsthour: Incomplete
        dstminute: Incomplete
        def __init__(self, path: str | None) -> None: ...

    def valuesToDict(key: winreg._KeyType) -> dict[str, Any]: ...



from _typeshed import FileDescriptorOrPath, Incomplete, ReadableBuffer
from collections.abc import Mapping, MutableMapping
from logging import Logger
from typing import Final
from typing_extensions import Self

INDEX_NAME: Final[str]
INDEX_URL: Final[str]
TOKEN_USERNAME: Final[str]
log: Logger

def resolve_repository_name(repo_name: str) -> tuple[str, str]: ...
def resolve_index_name(index_name: str) -> str: ...
def get_config_header(client, registry) -> bytes | None: ...
def split_repo_name(repo_name: str) -> tuple[str, str]: ...
def get_credential_store(authconfig: AuthConfig | MutableMapping[str, Incomplete], registry: str | None): ...

class AuthConfig(dict[str, Incomplete]):
    def __init__(self, dct: MutableMapping[str, Incomplete], credstore_env=None) -> None: ...
    @classmethod
    def parse_auth(
        cls, entries: Mapping[str, dict[Incomplete, Incomplete]], raise_on_error: bool = False
    ) -> dict[str, Incomplete]: ...
    @classmethod
    def load_config(
        cls, config_path: FileDescriptorOrPath | None, config_dict: dict[str, Incomplete] | None, credstore_env=None
    ) -> Self: ...
    @property
    def auths(self) -> dict[str, Incomplete]: ...
    @property
    def creds_store(self): ...
    @property
    def cred_helpers(self): ...
    @property
    def is_empty(self) -> bool: ...
    def resolve_authconfig(self, registry: str | None = None): ...
    def get_credential_store(self, registry: str | None): ...
    def get_all_credentials(self): ...
    def add_auth(self, reg: str, data) -> None: ...

def resolve_authconfig(authconfig, registry: str | None = None, credstore_env=None): ...
def convert_to_hostname(url: str) -> str: ...
def decode_auth(auth: str | ReadableBuffer) -> tuple[str, str]: ...
def encode_header(auth) -> bytes: ...
def parse_auth(entries: Mapping[str, dict[Incomplete, Incomplete]], raise_on_error: bool = False): ...
def load_config(
    config_path: FileDescriptorOrPath | None = None, config_dict: dict[str, Incomplete] | None = None, credstore_env=None
) -> AuthConfig: ...



from _typeshed import Incomplete
from typing import Any, NoReturn

from oauthlib.common import Request

class OAuth2Error(Exception):
    error: str | None
    status_code: int
    description: str
    uri: str | None
    state: str | None
    redirect_uri: str | None
    client_id: str | None
    scopes: Any
    response_type: str | None
    response_mode: str | None
    grant_type: str | None
    def __init__(
        self,
        description: str | None = None,
        uri: str | None = None,
        state: str | None = None,
        status_code: int | None = None,
        request: Request | None = None,
    ) -> None: ...
    def in_uri(self, uri: str) -> str: ...
    @property
    def twotuples(self) -> list[tuple[str, str | None]]: ...
    @property
    def urlencoded(self) -> str: ...
    @property
    def json(self) -> str: ...
    @property
    def headers(self) -> dict[str, str]: ...

class TokenExpiredError(OAuth2Error):
    error: str

class InsecureTransportError(OAuth2Error):
    error: str
    description: str

class MismatchingStateError(OAuth2Error):
    error: str
    description: str

class MissingCodeError(OAuth2Error):
    error: str

class MissingTokenError(OAuth2Error):
    error: str

class MissingTokenTypeError(OAuth2Error):
    error: str

class FatalClientError(OAuth2Error): ...

class InvalidRequestFatalError(FatalClientError):
    error: str

class InvalidRedirectURIError(InvalidRequestFatalError):
    description: str

class MissingRedirectURIError(InvalidRequestFatalError):
    description: str

class MismatchingRedirectURIError(InvalidRequestFatalError):
    description: str

class InvalidClientIdError(InvalidRequestFatalError):
    description: str

class MissingClientIdError(InvalidRequestFatalError):
    description: str

class InvalidRequestError(OAuth2Error):
    error: str

class MissingResponseTypeError(InvalidRequestError):
    description: str

class MissingCodeChallengeError(InvalidRequestError):
    description: str

class MissingCodeVerifierError(InvalidRequestError):
    description: str

class AccessDeniedError(OAuth2Error):
    error: str

class UnsupportedResponseTypeError(OAuth2Error):
    error: str

class UnsupportedCodeChallengeMethodError(InvalidRequestError):
    description: str

class InvalidScopeError(OAuth2Error):
    error: str

class ServerError(OAuth2Error):
    error: str

class TemporarilyUnavailableError(OAuth2Error):
    error: str

class InvalidClientError(FatalClientError):
    error: str
    status_code: int

class InvalidGrantError(OAuth2Error):
    error: str
    status_code: int

class UnauthorizedClientError(OAuth2Error):
    error: str

class UnsupportedGrantTypeError(OAuth2Error):
    error: str

class UnsupportedTokenTypeError(OAuth2Error):
    error: str

class InvalidTokenError(OAuth2Error):
    error: str
    status_code: int
    description: str

class InsufficientScopeError(OAuth2Error):
    error: str
    status_code: int
    description: str

class ConsentRequired(OAuth2Error):
    error: str

class LoginRequired(OAuth2Error):
    error: str

class CustomOAuth2Error(OAuth2Error):
    def __init__(
        self,
        error: str,
        description: str | None = None,
        uri: str | None = None,
        state: str | None = None,
        status_code: int | None = None,
        request: Request | None = None,
    ) -> None: ...

def raise_from_error(error: str, params: dict[str, Incomplete] | None = None) -> NoReturn: ...



from _typeshed import Incomplete
from logging import Logger

from oauthlib.common import Request, _HTTPMethod

from .base import BaseEndpoint

log: Logger

class ResourceEndpoint(BaseEndpoint):
    def __init__(self, default_token: str, token_types: dict[str, Incomplete]) -> None: ...
    @property
    def default_token(self) -> str: ...
    @property
    def default_token_type_handler(self): ...
    @property
    def tokens(self) -> dict[str, Incomplete]: ...
    def verify_request(
        self,
        uri: str,
        http_method: _HTTPMethod = "GET",
        body: str | None = None,
        headers: dict[str, str] | None = None,
        scopes=None,
    ) -> tuple[bool, Request]: ...
    def find_token_type(self, request: Request): ...



import sys
from collections.abc import Generator
from typing import Any

from pygments.filter import Filter
from pygments.formatter import Formatter
from pygments.lexer import Lexer
from pygments.style import Style

LEXER_ENTRY_POINT: str
FORMATTER_ENTRY_POINT: str
STYLE_ENTRY_POINT: str
FILTER_ENTRY_POINT: str

if sys.version_info >= (3, 10):
    from importlib.metadata import EntryPoints
    def iter_entry_points(group_name: str) -> EntryPoints: ...

else:
    from importlib.metadata import EntryPoint

    def iter_entry_points(group_name: str) -> tuple[EntryPoint, ...] | list[EntryPoint]: ...

def find_plugin_lexers() -> Generator[type[Lexer], None, None]: ...
def find_plugin_formatters() -> Generator[tuple[str, type[Formatter[Any]]], None, None]: ...
def find_plugin_styles() -> Generator[tuple[str, type[Style]], None, None]: ...
def find_plugin_filters() -> Generator[tuple[str, type[Filter]], None, None]: ...



from __future__ import annotations

import io
import sys
from tempfile import TemporaryFile, _TemporaryFileWrapper
from typing_extensions import assert_type

if sys.platform == "win32":
    assert_type(TemporaryFile(), _TemporaryFileWrapper[bytes])
    assert_type(TemporaryFile("w+"), _TemporaryFileWrapper[str])
    assert_type(TemporaryFile("w+b"), _TemporaryFileWrapper[bytes])
    assert_type(TemporaryFile("wb"), _TemporaryFileWrapper[bytes])
    assert_type(TemporaryFile("rb"), _TemporaryFileWrapper[bytes])
    assert_type(TemporaryFile("wb", 0), _TemporaryFileWrapper[bytes])
    assert_type(TemporaryFile(mode="w+"), _TemporaryFileWrapper[str])
    assert_type(TemporaryFile(mode="w+b"), _TemporaryFileWrapper[bytes])
    assert_type(TemporaryFile(mode="wb"), _TemporaryFileWrapper[bytes])
    assert_type(TemporaryFile(mode="rb"), _TemporaryFileWrapper[bytes])
    assert_type(TemporaryFile(buffering=0), _TemporaryFileWrapper[bytes])
else:
    assert_type(TemporaryFile(), io.BufferedRandom)
    assert_type(TemporaryFile("w+"), io.TextIOWrapper)
    assert_type(TemporaryFile("w+b"), io.BufferedRandom)
    assert_type(TemporaryFile("wb"), io.BufferedWriter)
    assert_type(TemporaryFile("rb"), io.BufferedReader)
    assert_type(TemporaryFile("wb", 0), io.FileIO)
    assert_type(TemporaryFile(mode="w+"), io.TextIOWrapper)
    assert_type(TemporaryFile(mode="w+b"), io.BufferedRandom)
    assert_type(TemporaryFile(mode="wb"), io.BufferedWriter)
    assert_type(TemporaryFile(mode="rb"), io.BufferedReader)
    assert_type(TemporaryFile(buffering=0), io.FileIO)



class Mark:
    name: str
    index: int
    line: int
    column: int
    buffer: str | None
    pointer: int
    def __init__(self, name: str, index: int, line: int, column: int, buffer: str | None, pointer: int) -> None: ...
    def get_snippet(self, indent: int = 4, max_length: int = 75) -> str | None: ...

class YAMLError(Exception): ...

class MarkedYAMLError(YAMLError):
    context: str | None
    context_mark: Mark | None
    problem: str | None
    problem_mark: Mark | None
    note: str | None
    def __init__(
        self,
        context: str | None = None,
        context_mark: Mark | None = None,
        problem: str | None = None,
        problem_mark: Mark | None = None,
        note: str | None = None,
    ) -> None: ...

__all__ = ["Mark", "YAMLError", "MarkedYAMLError"]



from collections.abc import Generator
from typing import NamedTuple

from .violation import Violation

class Statistics:
    def __init__(self) -> None: ...
    def error_codes(self) -> list[str]: ...
    def record(self, error: Violation) -> None: ...
    def statistics_for(self, prefix: str, filename: str | None = None) -> Generator[Statistic, None, None]: ...

class Key(NamedTuple):
    filename: str
    code: str
    @classmethod
    def create_from(cls, error: Violation) -> Key: ...
    def matches(self, prefix: str, filename: str | None) -> bool: ...

class Statistic:
    error_code: str
    filename: str
    message: str
    count: int
    def __init__(self, error_code: str, filename: str, message: str, count: int) -> None: ...
    @classmethod
    def create_from(cls, error: Violation) -> Statistic: ...
    def increment(self) -> None: ...



from typing import ClassVar, Final

from docutils import nodes
from docutils.transforms import Transform

__docformat__: Final = "reStructuredText"

class CallBack(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class ClassAttribute(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...

class Transitions(Transform):
    default_priority: ClassVar[int]
    def apply(self) -> None: ...
    def visit_transition(self, node: nodes.transition) -> None: ...



import abc
from _typeshed import Incomplete
from logging import Logger

import pika.compat
from pika.adapters.base_connection import BaseConnection
from pika.adapters.utils.selector_ioloop_adapter import AbstractSelectorIOLoop

LOGGER: Logger
SELECT_TYPE: Incomplete

class SelectConnection(BaseConnection):
    def __init__(
        self,
        parameters=None,
        on_open_callback=None,
        on_open_error_callback=None,
        on_close_callback=None,
        custom_ioloop=None,
        internal_connection_workflow: bool = True,
    ) -> None: ...
    @classmethod
    def create_connection(cls, connection_configs, on_done, custom_ioloop=None, workflow=None): ...

class _Timeout:
    __slots__ = ("deadline", "callback")
    deadline: Incomplete
    callback: Incomplete
    def __init__(self, deadline, callback) -> None: ...
    def __eq__(self, other): ...
    def __ne__(self, other): ...
    def __lt__(self, other): ...
    def __gt__(self, other): ...
    def __le__(self, other): ...
    def __ge__(self, other): ...

class _Timer:
    def __init__(self) -> None: ...
    def close(self) -> None: ...
    def call_later(self, delay, callback): ...
    def remove_timeout(self, timeout) -> None: ...
    def get_remaining_interval(self): ...
    def process_timeouts(self) -> None: ...

class PollEvents:
    READ: Incomplete
    WRITE: Incomplete
    ERROR: Incomplete

class IOLoop(AbstractSelectorIOLoop):
    READ: Incomplete
    WRITE: Incomplete
    ERROR: Incomplete
    def __init__(self) -> None: ...
    def close(self) -> None: ...
    def call_later(self, delay, callback): ...
    def remove_timeout(self, timeout_handle) -> None: ...
    def add_callback_threadsafe(self, callback) -> None: ...
    add_callback: Incomplete
    def process_timeouts(self) -> None: ...
    def add_handler(self, fd, handler, events) -> None: ...
    def update_handler(self, fd, events) -> None: ...
    def remove_handler(self, fd) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...
    def activate_poller(self) -> None: ...
    def deactivate_poller(self) -> None: ...
    def poll(self) -> None: ...

class _PollerBase(pika.compat.AbstractBase, metaclass=abc.ABCMeta):
    POLL_TIMEOUT_MULT: int
    def __init__(self, get_wait_seconds, process_timeouts) -> None: ...
    def close(self) -> None: ...
    def wake_threadsafe(self) -> None: ...
    def add_handler(self, fileno, handler, events) -> None: ...
    def update_handler(self, fileno, events) -> None: ...
    def remove_handler(self, fileno) -> None: ...
    def activate_poller(self) -> None: ...
    def deactivate_poller(self) -> None: ...
    def start(self) -> None: ...
    def stop(self) -> None: ...
    @abc.abstractmethod
    def poll(self): ...

class SelectPoller(_PollerBase):
    POLL_TIMEOUT_MULT: int
    def poll(self) -> None: ...

class KQueuePoller(_PollerBase):
    def __init__(self, get_wait_seconds, process_timeouts) -> None: ...
    def poll(self) -> None: ...

class PollPoller(_PollerBase):
    POLL_TIMEOUT_MULT: int
    def __init__(self, get_wait_seconds, process_timeouts) -> None: ...
    def poll(self) -> None: ...

class EPollPoller(PollPoller):
    POLL_TIMEOUT_MULT: int



from _typeshed import Incomplete
from collections.abc import Sequence
from logging import Logger

LOG: Logger

class Application:
    start_time: Incomplete
    end_time: Incomplete
    plugins: Incomplete
    formatter: Incomplete
    guide: Incomplete
    file_checker_manager: Incomplete
    options: Incomplete
    result_count: int
    total_result_count: int
    catastrophic_failure: bool
    def __init__(self) -> None: ...
    def exit_code(self) -> int: ...
    def make_formatter(self) -> None: ...
    def make_guide(self) -> None: ...
    def make_file_checker_manager(self, argv: Sequence[str]) -> None: ...
    def run_checks(self) -> None: ...
    def report_benchmarks(self) -> None: ...
    def report_errors(self) -> None: ...
    def report_statistics(self) -> None: ...
    def initialize(self, argv: Sequence[str]) -> None: ...
    def report(self) -> None: ...
    def run(self, argv: Sequence[str]) -> None: ...



import datetime

from .autumn_holiday import (
    AutumnHolidayFirstMondayOctober as AutumnHolidayFirstMondayOctober,
    AutumnHolidayLastMondaySeptember as AutumnHolidayLastMondaySeptember,
    AutumnHolidaySecondMondayOctober as AutumnHolidaySecondMondayOctober,
    AutumnHolidayThirdMondayOctober as AutumnHolidayThirdMondayOctober,
)
from .fair_holiday import (
    FairHolidayFirstMondayAugust as FairHolidayFirstMondayAugust,
    FairHolidayFirstMondayJuly as FairHolidayFirstMondayJuly,
    FairHolidayFourthFridayJuly as FairHolidayFourthFridayJuly,
    FairHolidayLastMondayJuly as FairHolidayLastMondayJuly,
    FairHolidayLastMondayJune as FairHolidayLastMondayJune,
    FairHolidaySecondMondayJuly as FairHolidaySecondMondayJuly,
    FairHolidayThirdMondayJuly as FairHolidayThirdMondayJuly,
)
from .spring_holiday import (
    SpringHolidayFirstMondayApril as SpringHolidayFirstMondayApril,
    SpringHolidayFirstMondayJune as SpringHolidayFirstMondayJune,
    SpringHolidayLastMondayMay as SpringHolidayLastMondayMay,
    SpringHolidaySecondMondayApril as SpringHolidaySecondMondayApril,
    SpringHolidayTuesdayAfterFirstMondayMay as SpringHolidayTuesdayAfterFirstMondayMay,
)
from .victoria_day import (
    VictoriaDayFirstMondayJune as VictoriaDayFirstMondayJune,
    VictoriaDayFourthMondayMay as VictoriaDayFourthMondayMay,
    VictoriaDayLastMondayMay as VictoriaDayLastMondayMay,
)

class LateSummer:
    def get_variable_days(self, year: int) -> list[tuple[datetime.date, str]]: ...

class BattleStirlingBridge:
    def get_variable_days(self, year: int) -> list[tuple[datetime.date, str]]: ...

class AyrGoldCup:
    def get_variable_days(self, year: int) -> list[tuple[datetime.date, str]]: ...

# Names in __all__ with no definition:
#   VictoriaDayTuesdayAfterFirstMondayMay

__all__ = [
    "AyrGoldCup",
    "SpringHolidayFirstMondayApril",
    "SpringHolidaySecondMondayApril",
    "SpringHolidayTuesdayAfterFirstMondayMay",
    "SpringHolidayLastMondayMay",
    "SpringHolidayFirstMondayJune",
    "VictoriaDayFourthMondayMay",
    "VictoriaDayLastMondayMay",
    "VictoriaDayTuesdayAfterFirstMondayMay",  # noqa: F822 # pyright: ignore[reportUnsupportedDunderAll] see https://github.com/workalendar/workalendar/pull/778
    "VictoriaDayFirstMondayJune",
    "FairHolidayLastMondayJune",
    "FairHolidayFirstMondayJuly",
    "FairHolidaySecondMondayJuly",
    "FairHolidayThirdMondayJuly",
    "FairHolidayLastMondayJuly",
    "FairHolidayFourthFridayJuly",
    "FairHolidayFirstMondayAugust",
    "AutumnHolidayLastMondaySeptember",
    "AutumnHolidayFirstMondayOctober",
    "AutumnHolidaySecondMondayOctober",
    "AutumnHolidayThirdMondayOctober",
]



import asyncio
from _typeshed import OptExcInfo
from asyncio import BaseEventLoop
from collections.abc import Callable, Coroutine
from concurrent.futures import ThreadPoolExecutor
from typing import Any, TypeVar
from typing_extensions import ParamSpec

from asgiref.sync import SyncToAsync

_P = ParamSpec("_P")
_R = TypeVar("_R")

class DatabaseSyncToAsync(SyncToAsync[_P, _R]):
    def thread_handler(
        self,
        loop: BaseEventLoop,
        exc_info: OptExcInfo,
        task_context: list[asyncio.Task[Any]] | None,
        func: Callable[_P, _R],
        *args: _P.args,
        **kwargs: _P.kwargs,
    ) -> _R: ...

# We define `database_sync_to_async` as a function instead of assigning
# `DatabaseSyncToAsync(...)` directly, to preserve both decorator and
# higher-order function behavior with correct type hints.
# A direct assignment would result in incorrect type inference for the wrapped function.
def database_sync_to_async(
    func: Callable[_P, _R], thread_sensitive: bool = True, executor: ThreadPoolExecutor | None = None
) -> Callable[_P, Coroutine[Any, Any, _R]]: ...
async def aclose_old_connections() -> None: ...



from typing import Final

from . import (
    armenian as armenian,
    bahai as bahai,
    coptic as coptic,
    daycount as daycount,
    dublin as dublin,
    french_republican as french_republican,
    gregorian as gregorian,
    hebrew as hebrew,
    holidays as holidays,
    indian_civil as indian_civil,
    islamic as islamic,
    iso as iso,
    julian as julian,
    julianday as julianday,
    mayan as mayan,
    ordinal as ordinal,
    persian as persian,
    positivist as positivist,
    utils as utils,
)

__version__: Final[str]

__all__ = [
    "holidays",
    "armenian",
    "bahai",
    "coptic",
    "dublin",
    "daycount",
    "french_republican",
    "gregorian",
    "hebrew",
    "indian_civil",
    "islamic",
    "iso",
    "julian",
    "julianday",
    "mayan",
    "persian",
    "positivist",
    "mayan",
    "ordinal",
    "utils",
]



from networkx.utils.backends import _dispatchable

__all__ = ["uniform_random_intersection_graph", "k_random_intersection_graph", "general_random_intersection_graph"]

@_dispatchable
def uniform_random_intersection_graph(n, m, p, seed=None): ...
@_dispatchable
def k_random_intersection_graph(n, m, k, seed=None): ...
@_dispatchable
def general_random_intersection_graph(n, m, p, seed=None): ...



from _typeshed import Incomplete

import win32com.server.dispatcher

debugging: int

def trace(*args) -> None: ...
def RaiseNotImpl(who: Incomplete | None = ...) -> None: ...

class Dispatcher(win32com.server.dispatcher.DispatcherWin32trace):
    def __init__(self, policyClass, object) -> None: ...



from _typeshed import Incomplete

from antlr4.tree.ParseTreePattern import ParseTreePattern as ParseTreePattern
from antlr4.tree.Tree import ParseTree as ParseTree

class ParseTreeMatch:
    __slots__ = ("tree", "pattern", "labels", "mismatchedNode")
    tree: Incomplete
    pattern: Incomplete
    labels: Incomplete
    mismatchedNode: Incomplete
    def __init__(
        self, tree: ParseTree, pattern: ParseTreePattern, labels: dict[str, list[ParseTree]], mismatchedNode: ParseTree
    ) -> None: ...
    def get(self, label: str): ...
    def getAll(self, label: str): ...
    def succeeded(self): ...



from _typeshed import Incomplete
from collections.abc import Generator

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = [
    "bidirectional_shortest_path",
    "single_source_shortest_path",
    "single_source_shortest_path_length",
    "single_target_shortest_path",
    "single_target_shortest_path_length",
    "all_pairs_shortest_path",
    "all_pairs_shortest_path_length",
    "predecessor",
]

@_dispatchable
def single_source_shortest_path_length(G: Graph[_Node], source: _Node, cutoff: int | None = None) -> dict[Incomplete, int]: ...
@_dispatchable
def single_target_shortest_path_length(G: Graph[_Node], target: _Node, cutoff: int | None = None): ...
@_dispatchable
def all_pairs_shortest_path_length(G: Graph[_Node], cutoff: int | None = None) -> Generator[Incomplete, None, None]: ...
@_dispatchable
def bidirectional_shortest_path(G: Graph[_Node], source: _Node, target: _Node) -> list[Incomplete]: ...
@_dispatchable
def single_source_shortest_path(
    G: Graph[_Node], source: _Node, cutoff: int | None = None
) -> dict[Incomplete, list[Incomplete]]: ...
@_dispatchable
def single_target_shortest_path(
    G: Graph[_Node], target: _Node, cutoff: int | None = None
) -> dict[Incomplete, list[Incomplete]]: ...
@_dispatchable
def all_pairs_shortest_path(
    G: Graph[_Node], cutoff: int | None = None
) -> Generator[tuple[Incomplete, dict[Incomplete, list[Incomplete]]]]: ...
@_dispatchable
def predecessor(
    G: Graph[_Node], source: _Node, target: _Node | None = None, cutoff: int | None = None, return_seen: bool | None = None
): ...



from _typeshed import Incomplete

from antlr4.atn.ATN import ATN as ATN
from antlr4.atn.ATNConfigSet import ATNConfigSet as ATNConfigSet
from antlr4.dfa.DFAState import DFAState as DFAState
from antlr4.PredictionContext import (
    PredictionContext as PredictionContext,
    PredictionContextCache as PredictionContextCache,
    getCachedPredictionContext as getCachedPredictionContext,
)

class ATNSimulator:
    __slots__ = ("atn", "sharedContextCache", "__dict__")
    ERROR: Incomplete
    atn: Incomplete
    sharedContextCache: Incomplete
    def __init__(self, atn: ATN, sharedContextCache: PredictionContextCache) -> None: ...
    def getCachedContext(self, context: PredictionContext): ...



from collections.abc import Callable, Iterable
from enum import Enum
from typing import TypeVar, overload
from typing_extensions import ParamSpec

import tensorflow as tf
from tensorflow._aliases import Integer

_Param = ParamSpec("_Param")
_RetType = TypeVar("_RetType")

class Feature(Enum):
    ALL = "ALL"
    ASSERT_STATEMENTS = "ASSERT_STATEMENTS"
    AUTO_CONTROL_DEPS = "AUTO_CONTROL_DEPS"
    BUILTIN_FUNCTIONS = "BUILTIN_FUNCTIONS"
    EQUALITY_OPERATORS = "EQUALITY_OPERATORS"
    LISTS = "LISTS"
    NAME_SCOPES = "NAME_SCOPES"

@overload
def do_not_convert(func: Callable[_Param, _RetType]) -> Callable[_Param, _RetType]: ...
@overload
def do_not_convert(func: None = None) -> Callable[[Callable[_Param, _RetType]], Callable[_Param, _RetType]]: ...
def set_loop_options(
    parallel_iterations: Integer = ...,
    swap_memory: bool = ...,
    maximum_iterations: Integer = ...,
    shape_invariants: Iterable[tuple[tf.Tensor, tf.TensorShape]] = ...,
) -> None: ...



from antlr4.InputStream import InputStream
from antlr4.Lexer import TokenSource

class Token:
    __slots__ = ("source", "type", "channel", "start", "stop", "tokenIndex", "line", "column", "_text")
    INVALID_TYPE: int
    EPSILON: int
    MIN_USER_TOKEN_TYPE: int
    EOF: int
    DEFAULT_CHANNEL: int
    HIDDEN_CHANNEL: int
    source: tuple[TokenSource | None, InputStream | None]
    type: int
    channel: int
    start: int
    stop: int
    tokenIndex: int | None
    line: int
    column: int
    def __init__(self) -> None: ...
    @property
    def text(self) -> str: ...
    @text.setter
    def text(self, text: str) -> None: ...
    def getTokenSource(self) -> TokenSource | None: ...
    def getInputStream(self) -> InputStream | None: ...

class CommonToken(Token):
    EMPTY_SOURCE: tuple[None, None]
    source: tuple[TokenSource | None, InputStream | None]
    type: int
    channel: int
    start: int
    stop: int
    tokenIndex: int
    line: int
    column: int
    def __init__(
        self,
        source: tuple[TokenSource | None, InputStream | None] = (None, None),
        type: int | None = None,
        channel: int = 0,
        start: int = -1,
        stop: int = -1,
    ) -> None: ...
    def clone(self) -> CommonToken: ...
    @property
    def text(self) -> str: ...
    @text.setter
    def text(self, text: str) -> None: ...



from aws_xray_sdk.ext.dbapi2 import XRayTracedConn

def patch() -> None: ...

class XRayTracedSQLite(XRayTracedConn):
    def execute(self, *args, **kwargs): ...
    def executemany(self, *args, **kwargs): ...



import sys
from _collections_abc import dict_keys
from _typeshed import FileDescriptorOrPath, ReadableBuffer, SupportsRead, SupportsWrite
from collections.abc import Callable, Generator, ItemsView, Iterable, Iterator, Mapping, Sequence
from typing import Any, Final, Generic, Literal, Protocol, SupportsIndex, TypeVar, overload, type_check_only
from typing_extensions import TypeAlias, TypeGuard, deprecated, disjoint_base
from xml.parsers.expat import XMLParserType

__all__ = [
    "C14NWriterTarget",
    "Comment",
    "dump",
    "Element",
    "ElementTree",
    "canonicalize",
    "fromstring",
    "fromstringlist",
    "indent",
    "iselement",
    "iterparse",
    "parse",
    "ParseError",
    "PI",
    "ProcessingInstruction",
    "QName",
    "SubElement",
    "tostring",
    "tostringlist",
    "TreeBuilder",
    "VERSION",
    "XML",
    "XMLID",
    "XMLParser",
    "XMLPullParser",
    "register_namespace",
]

_T = TypeVar("_T")
_FileRead: TypeAlias = FileDescriptorOrPath | SupportsRead[bytes] | SupportsRead[str]
_FileWriteC14N: TypeAlias = FileDescriptorOrPath | SupportsWrite[bytes]
_FileWrite: TypeAlias = _FileWriteC14N | SupportsWrite[str]

VERSION: Final[str]

class ParseError(SyntaxError):
    code: int
    position: tuple[int, int]

# In reality it works based on `.tag` attribute duck typing.
def iselement(element: object) -> TypeGuard[Element]: ...
@overload
def canonicalize(
    xml_data: str | ReadableBuffer | None = None,
    *,
    out: None = None,
    from_file: _FileRead | None = None,
    with_comments: bool = False,
    strip_text: bool = False,
    rewrite_prefixes: bool = False,
    qname_aware_tags: Iterable[str] | None = None,
    qname_aware_attrs: Iterable[str] | None = None,
    exclude_attrs: Iterable[str] | None = None,
    exclude_tags: Iterable[str] | None = None,
) -> str: ...
@overload
def canonicalize(
    xml_data: str | ReadableBuffer | None = None,
    *,
    out: SupportsWrite[str],
    from_file: _FileRead | None = None,
    with_comments: bool = False,
    strip_text: bool = False,
    rewrite_prefixes: bool = False,
    qname_aware_tags: Iterable[str] | None = None,
    qname_aware_attrs: Iterable[str] | None = None,
    exclude_attrs: Iterable[str] | None = None,
    exclude_tags: Iterable[str] | None = None,
) -> None: ...

# The tag for Element can be set to the Comment or ProcessingInstruction
# functions defined in this module.
_ElementCallable: TypeAlias = Callable[..., Element[_ElementCallable]]

_Tag = TypeVar("_Tag", default=str, bound=str | _ElementCallable)
_OtherTag = TypeVar("_OtherTag", default=str, bound=str | _ElementCallable)

@disjoint_base
class Element(Generic[_Tag]):
    tag: _Tag
    attrib: dict[str, str]
    text: str | None
    tail: str | None
    def __init__(self, tag: _Tag, attrib: dict[str, str] = {}, **extra: str) -> None: ...
    def append(self, subelement: Element[Any], /) -> None: ...
    def clear(self) -> None: ...
    def extend(self, elements: Iterable[Element[Any]], /) -> None: ...
    def find(self, path: str, namespaces: dict[str, str] | None = None) -> Element | None: ...
    def findall(self, path: str, namespaces: dict[str, str] | None = None) -> list[Element]: ...
    @overload
    def findtext(self, path: str, default: None = None, namespaces: dict[str, str] | None = None) -> str | None: ...
    @overload
    def findtext(self, path: str, default: _T, namespaces: dict[str, str] | None = None) -> _T | str: ...
    @overload
    def get(self, key: str, default: None = None) -> str | None: ...
    @overload
    def get(self, key: str, default: _T) -> str | _T: ...
    def insert(self, index: int, subelement: Element[Any], /) -> None: ...
    def items(self) -> ItemsView[str, str]: ...
    def iter(self, tag: str | None = None) -> Generator[Element, None, None]: ...
    @overload
    def iterfind(self, path: Literal[""], namespaces: dict[str, str] | None = None) -> None: ...  # type: ignore[overload-overlap]
    @overload
    def iterfind(self, path: str, namespaces: dict[str, str] | None = None) -> Generator[Element, None, None]: ...
    def itertext(self) -> Generator[str, None, None]: ...
    def keys(self) -> dict_keys[str, str]: ...
    # makeelement returns the type of self in Python impl, but not in C impl
    def makeelement(self, tag: _OtherTag, attrib: dict[str, str], /) -> Element[_OtherTag]: ...
    def remove(self, subelement: Element[Any], /) -> None: ...
    def set(self, key: str, value: str, /) -> None: ...
    def __copy__(self) -> Element[_Tag]: ...  # returns the type of self in Python impl, but not in C impl
    def __deepcopy__(self, memo: Any, /) -> Element: ...  # Only exists in C impl
    def __delitem__(self, key: SupportsIndex | slice, /) -> None: ...
    @overload
    def __getitem__(self, key: SupportsIndex, /) -> Element: ...
    @overload
    def __getitem__(self, key: slice, /) -> list[Element]: ...
    def __len__(self) -> int: ...
    # Doesn't actually exist at runtime, but instance of the class are indeed iterable due to __getitem__.
    def __iter__(self) -> Iterator[Element]: ...
    @overload
    def __setitem__(self, key: SupportsIndex, value: Element[Any], /) -> None: ...
    @overload
    def __setitem__(self, key: slice, value: Iterable[Element[Any]], /) -> None: ...

    # Doesn't really exist in earlier versions, where __len__ is called implicitly instead
    @deprecated("Testing an element's truth value is deprecated.")
    def __bool__(self) -> bool: ...

def SubElement(parent: Element[Any], tag: str, attrib: dict[str, str] = ..., **extra: str) -> Element: ...
def Comment(text: str | None = None) -> Element[_ElementCallable]: ...
def ProcessingInstruction(target: str, text: str | None = None) -> Element[_ElementCallable]: ...

PI = ProcessingInstruction

class QName:
    text: str
    def __init__(self, text_or_uri: str, tag: str | None = None) -> None: ...
    def __lt__(self, other: QName | str) -> bool: ...
    def __le__(self, other: QName | str) -> bool: ...
    def __gt__(self, other: QName | str) -> bool: ...
    def __ge__(self, other: QName | str) -> bool: ...
    def __eq__(self, other: object) -> bool: ...
    def __hash__(self) -> int: ...

_Root = TypeVar("_Root", Element, Element | None, default=Element | None)

class ElementTree(Generic[_Root]):
    def __init__(self, element: Element[Any] | None = None, file: _FileRead | None = None) -> None: ...
    def getroot(self) -> _Root: ...
    def parse(self, source: _FileRead, parser: XMLParser | None = None) -> Element: ...
    def iter(self, tag: str | None = None) -> Generator[Element, None, None]: ...
    def find(self, path: str, namespaces: dict[str, str] | None = None) -> Element | None: ...
    @overload
    def findtext(self, path: str, default: None = None, namespaces: dict[str, str] | None = None) -> str | None: ...
    @overload
    def findtext(self, path: str, default: _T, namespaces: dict[str, str] | None = None) -> _T | str: ...
    def findall(self, path: str, namespaces: dict[str, str] | None = None) -> list[Element]: ...
    @overload
    def iterfind(self, path: Literal[""], namespaces: dict[str, str] | None = None) -> None: ...  # type: ignore[overload-overlap]
    @overload
    def iterfind(self, path: str, namespaces: dict[str, str] | None = None) -> Generator[Element, None, None]: ...
    def write(
        self,
        file_or_filename: _FileWrite,
        encoding: str | None = None,
        xml_declaration: bool | None = None,
        default_namespace: str | None = None,
        method: Literal["xml", "html", "text", "c14n"] | None = None,
        *,
        short_empty_elements: bool = True,
    ) -> None: ...
    def write_c14n(self, file: _FileWriteC14N) -> None: ...

HTML_EMPTY: Final[set[str]]

def register_namespace(prefix: str, uri: str) -> None: ...
@overload
def tostring(
    element: Element[Any],
    encoding: None = None,
    method: Literal["xml", "html", "text", "c14n"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
) -> bytes: ...
@overload
def tostring(
    element: Element[Any],
    encoding: Literal["unicode"],
    method: Literal["xml", "html", "text", "c14n"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
) -> str: ...
@overload
def tostring(
    element: Element[Any],
    encoding: str,
    method: Literal["xml", "html", "text", "c14n"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
) -> Any: ...
@overload
def tostringlist(
    element: Element[Any],
    encoding: None = None,
    method: Literal["xml", "html", "text", "c14n"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
) -> list[bytes]: ...
@overload
def tostringlist(
    element: Element[Any],
    encoding: Literal["unicode"],
    method: Literal["xml", "html", "text", "c14n"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
) -> list[str]: ...
@overload
def tostringlist(
    element: Element[Any],
    encoding: str,
    method: Literal["xml", "html", "text", "c14n"] | None = None,
    *,
    xml_declaration: bool | None = None,
    default_namespace: str | None = None,
    short_empty_elements: bool = True,
) -> list[Any]: ...
def dump(elem: Element[Any] | ElementTree[Any]) -> None: ...
def indent(tree: Element[Any] | ElementTree[Any], space: str = "  ", level: int = 0) -> None: ...
def parse(source: _FileRead, parser: XMLParser[Any] | None = None) -> ElementTree[Element]: ...

# This class is defined inside the body of iterparse
@type_check_only
class _IterParseIterator(Iterator[tuple[str, Element]], Protocol):
    def __next__(self) -> tuple[str, Element]: ...
    if sys.version_info >= (3, 13):
        def close(self) -> None: ...
    if sys.version_info >= (3, 11):
        def __del__(self) -> None: ...

def iterparse(source: _FileRead, events: Sequence[str] | None = None, parser: XMLParser | None = None) -> _IterParseIterator: ...

_EventQueue: TypeAlias = tuple[str] | tuple[str, tuple[str, str]] | tuple[str, None]

class XMLPullParser(Generic[_E]):
    def __init__(self, events: Sequence[str] | None = None, *, _parser: XMLParser[_E] | None = None) -> None: ...
    def feed(self, data: str | ReadableBuffer) -> None: ...
    def close(self) -> None: ...
    def read_events(self) -> Iterator[_EventQueue | tuple[str, _E]]: ...
    def flush(self) -> None: ...

def XML(text: str | ReadableBuffer, parser: XMLParser | None = None) -> Element: ...
def XMLID(text: str | ReadableBuffer, parser: XMLParser | None = None) -> tuple[Element, dict[str, Element]]: ...

# This is aliased to XML in the source.
fromstring = XML

def fromstringlist(sequence: Sequence[str | ReadableBuffer], parser: XMLParser | None = None) -> Element: ...

# This type is both not precise enough and too precise. The TreeBuilder
# requires the elementfactory to accept tag and attrs in its args and produce
# some kind of object that has .text and .tail properties.
# I've chosen to constrain the ElementFactory to always produce an Element
# because that is how almost everyone will use it.
# Unfortunately, the type of the factory arguments is dependent on how
# TreeBuilder is called by client code (they could pass strs, bytes or whatever);
# but we don't want to use a too-broad type, or it would be too hard to write
# elementfactories.
_ElementFactory: TypeAlias = Callable[[Any, dict[Any, Any]], Element]

@disjoint_base
class TreeBuilder:
    # comment_factory can take None because passing None to Comment is not an error
    def __init__(
        self,
        element_factory: _ElementFactory | None = None,
        *,
        comment_factory: Callable[[str | None], Element[Any]] | None = None,
        pi_factory: Callable[[str, str | None], Element[Any]] | None = None,
        insert_comments: bool = False,
        insert_pis: bool = False,
    ) -> None: ...
    insert_comments: bool
    insert_pis: bool

    def close(self) -> Element: ...
    def data(self, data: str, /) -> None: ...
    # tag and attrs are passed to the element_factory, so they could be anything
    # depending on what the particular factory supports.
    def start(self, tag: Any, attrs: dict[Any, Any], /) -> Element: ...
    def end(self, tag: str, /) -> Element: ...
    # These two methods have pos-only parameters in the C implementation
    def comment(self, text: str | None, /) -> Element[Any]: ...
    def pi(self, target: str, text: str | None = None, /) -> Element[Any]: ...

class C14NWriterTarget:
    def __init__(
        self,
        write: Callable[[str], object],
        *,
        with_comments: bool = False,
        strip_text: bool = False,
        rewrite_prefixes: bool = False,
        qname_aware_tags: Iterable[str] | None = None,
        qname_aware_attrs: Iterable[str] | None = None,
        exclude_attrs: Iterable[str] | None = None,
        exclude_tags: Iterable[str] | None = None,
    ) -> None: ...
    def data(self, data: str) -> None: ...
    def start_ns(self, prefix: str, uri: str) -> None: ...
    def start(self, tag: str, attrs: Mapping[str, str]) -> None: ...
    def end(self, tag: str) -> None: ...
    def comment(self, text: str) -> None: ...
    def pi(self, target: str, data: str) -> None: ...

# The target type is tricky, because the implementation doesn't
# require any particular attribute to be present. This documents the attributes
# that can be present, but uncommenting any of them would require them.
@type_check_only
class _Target(Protocol):
    # start: Callable[str, dict[str, str], Any] | None
    # end: Callable[[str], Any] | None
    # start_ns: Callable[[str, str], Any] | None
    # end_ns: Callable[[str], Any] | None
    # data: Callable[[str], Any] | None
    # comment: Callable[[str], Any]
    # pi: Callable[[str, str], Any] | None
    # close: Callable[[], Any] | None
    ...

_E = TypeVar("_E", default=Element)

# This is generic because the return type of close() depends on the target.
# The default target is TreeBuilder, which returns Element.
# C14NWriterTarget does not implement a close method, so using it results
# in a type of XMLParser[None].
@disjoint_base
class XMLParser(Generic[_E]):
    parser: XMLParserType
    target: _Target
    # TODO: what is entity used for???
    entity: dict[str, str]
    version: str
    def __init__(self, *, target: _Target | None = None, encoding: str | None = None) -> None: ...
    def close(self) -> _E: ...
    def feed(self, data: str | ReadableBuffer, /) -> None: ...
    def flush(self) -> None: ...



from typing import Final

ERROR_FIRST: Final = 1000
HASHCHK: Final = 1000
NISAMCHK: Final = 1001
NO: Final = 1002
YES: Final = 1003
CANT_CREATE_FILE: Final = 1004
CANT_CREATE_TABLE: Final = 1005
CANT_CREATE_DB: Final = 1006
DB_CREATE_EXISTS: Final = 1007
DB_DROP_EXISTS: Final = 1008
DB_DROP_DELETE: Final = 1009
DB_DROP_RMDIR: Final = 1010
CANT_DELETE_FILE: Final = 1011
CANT_FIND_SYSTEM_REC: Final = 1012
CANT_GET_STAT: Final = 1013
CANT_GET_WD: Final = 1014
CANT_LOCK: Final = 1015
CANT_OPEN_FILE: Final = 1016
FILE_NOT_FOUND: Final = 1017
CANT_READ_DIR: Final = 1018
CANT_SET_WD: Final = 1019
CHECKREAD: Final = 1020
DISK_FULL: Final = 1021
DUP_KEY: Final = 1022
ERROR_ON_CLOSE: Final = 1023
ERROR_ON_READ: Final = 1024
ERROR_ON_RENAME: Final = 1025
ERROR_ON_WRITE: Final = 1026
FILE_USED: Final = 1027
FILSORT_ABORT: Final = 1028
FORM_NOT_FOUND: Final = 1029
GET_ERRNO: Final = 1030
ILLEGAL_HA: Final = 1031
KEY_NOT_FOUND: Final = 1032
NOT_FORM_FILE: Final = 1033
NOT_KEYFILE: Final = 1034
OLD_KEYFILE: Final = 1035
OPEN_AS_READONLY: Final = 1036
OUTOFMEMORY: Final = 1037
OUT_OF_SORTMEMORY: Final = 1038
UNEXPECTED_EOF: Final = 1039
CON_COUNT_ERROR: Final = 1040
OUT_OF_RESOURCES: Final = 1041
BAD_HOST_ERROR: Final = 1042
HANDSHAKE_ERROR: Final = 1043
DBACCESS_DENIED_ERROR: Final = 1044
ACCESS_DENIED_ERROR: Final = 1045
NO_DB_ERROR: Final = 1046
UNKNOWN_COM_ERROR: Final = 1047
BAD_NULL_ERROR: Final = 1048
BAD_DB_ERROR: Final = 1049
TABLE_EXISTS_ERROR: Final = 1050
BAD_TABLE_ERROR: Final = 1051
NON_UNIQ_ERROR: Final = 1052
SERVER_SHUTDOWN: Final = 1053
BAD_FIELD_ERROR: Final = 1054
WRONG_FIELD_WITH_GROUP: Final = 1055
WRONG_GROUP_FIELD: Final = 1056
WRONG_SUM_SELECT: Final = 1057
WRONG_VALUE_COUNT: Final = 1058
TOO_LONG_IDENT: Final = 1059
DUP_FIELDNAME: Final = 1060
DUP_KEYNAME: Final = 1061
DUP_ENTRY: Final = 1062
WRONG_FIELD_SPEC: Final = 1063
PARSE_ERROR: Final = 1064
EMPTY_QUERY: Final = 1065
NONUNIQ_TABLE: Final = 1066
INVALID_DEFAULT: Final = 1067
MULTIPLE_PRI_KEY: Final = 1068
TOO_MANY_KEYS: Final = 1069
TOO_MANY_KEY_PARTS: Final = 1070
TOO_LONG_KEY: Final = 1071
KEY_COLUMN_DOES_NOT_EXITS: Final = 1072
BLOB_USED_AS_KEY: Final = 1073
TOO_BIG_FIELDLENGTH: Final = 1074
WRONG_AUTO_KEY: Final = 1075
READY: Final = 1076
NORMAL_SHUTDOWN: Final = 1077
GOT_SIGNAL: Final = 1078
SHUTDOWN_COMPLETE: Final = 1079
FORCING_CLOSE: Final = 1080
IPSOCK_ERROR: Final = 1081
NO_SUCH_INDEX: Final = 1082
WRONG_FIELD_TERMINATORS: Final = 1083
BLOBS_AND_NO_TERMINATED: Final = 1084
TEXTFILE_NOT_READABLE: Final = 1085
FILE_EXISTS_ERROR: Final = 1086
LOAD_INFO: Final = 1087
ALTER_INFO: Final = 1088
WRONG_SUB_KEY: Final = 1089
CANT_REMOVE_ALL_FIELDS: Final = 1090
CANT_DROP_FIELD_OR_KEY: Final = 1091
INSERT_INFO: Final = 1092
UPDATE_TABLE_USED: Final = 1093
NO_SUCH_THREAD: Final = 1094
KILL_DENIED_ERROR: Final = 1095
NO_TABLES_USED: Final = 1096
TOO_BIG_SET: Final = 1097
NO_UNIQUE_LOGFILE: Final = 1098
TABLE_NOT_LOCKED_FOR_WRITE: Final = 1099
TABLE_NOT_LOCKED: Final = 1100
BLOB_CANT_HAVE_DEFAULT: Final = 1101
WRONG_DB_NAME: Final = 1102
WRONG_TABLE_NAME: Final = 1103
TOO_BIG_SELECT: Final = 1104
UNKNOWN_ERROR: Final = 1105
UNKNOWN_PROCEDURE: Final = 1106
WRONG_PARAMCOUNT_TO_PROCEDURE: Final = 1107
WRONG_PARAMETERS_TO_PROCEDURE: Final = 1108
UNKNOWN_TABLE: Final = 1109
FIELD_SPECIFIED_TWICE: Final = 1110
INVALID_GROUP_FUNC_USE: Final = 1111
UNSUPPORTED_EXTENSION: Final = 1112
TABLE_MUST_HAVE_COLUMNS: Final = 1113
RECORD_FILE_FULL: Final = 1114
UNKNOWN_CHARACTER_SET: Final = 1115
TOO_MANY_TABLES: Final = 1116
TOO_MANY_FIELDS: Final = 1117
TOO_BIG_ROWSIZE: Final = 1118
STACK_OVERRUN: Final = 1119
WRONG_OUTER_JOIN: Final = 1120
NULL_COLUMN_IN_INDEX: Final = 1121
CANT_FIND_UDF: Final = 1122
CANT_INITIALIZE_UDF: Final = 1123
UDF_NO_PATHS: Final = 1124
UDF_EXISTS: Final = 1125
CANT_OPEN_LIBRARY: Final = 1126
CANT_FIND_DL_ENTRY: Final = 1127
FUNCTION_NOT_DEFINED: Final = 1128
HOST_IS_BLOCKED: Final = 1129
HOST_NOT_PRIVILEGED: Final = 1130
PASSWORD_ANONYMOUS_USER: Final = 1131
PASSWORD_NOT_ALLOWED: Final = 1132
PASSWORD_NO_MATCH: Final = 1133
UPDATE_INFO: Final = 1134
CANT_CREATE_THREAD: Final = 1135
WRONG_VALUE_COUNT_ON_ROW: Final = 1136
CANT_REOPEN_TABLE: Final = 1137
INVALID_USE_OF_NULL: Final = 1138
REGEXP_ERROR: Final = 1139
MIX_OF_GROUP_FUNC_AND_FIELDS: Final = 1140
NONEXISTING_GRANT: Final = 1141
TABLEACCESS_DENIED_ERROR: Final = 1142
COLUMNACCESS_DENIED_ERROR: Final = 1143
ILLEGAL_GRANT_FOR_TABLE: Final = 1144
GRANT_WRONG_HOST_OR_USER: Final = 1145
NO_SUCH_TABLE: Final = 1146
NONEXISTING_TABLE_GRANT: Final = 1147
NOT_ALLOWED_COMMAND: Final = 1148
SYNTAX_ERROR: Final = 1149
DELAYED_CANT_CHANGE_LOCK: Final = 1150
TOO_MANY_DELAYED_THREADS: Final = 1151
ABORTING_CONNECTION: Final = 1152
NET_PACKET_TOO_LARGE: Final = 1153
NET_READ_ERROR_FROM_PIPE: Final = 1154
NET_FCNTL_ERROR: Final = 1155
NET_PACKETS_OUT_OF_ORDER: Final = 1156
NET_UNCOMPRESS_ERROR: Final = 1157
NET_READ_ERROR: Final = 1158
NET_READ_INTERRUPTED: Final = 1159
NET_ERROR_ON_WRITE: Final = 1160
NET_WRITE_INTERRUPTED: Final = 1161
TOO_LONG_STRING: Final = 1162
TABLE_CANT_HANDLE_BLOB: Final = 1163
TABLE_CANT_HANDLE_AUTO_INCREMENT: Final = 1164
DELAYED_INSERT_TABLE_LOCKED: Final = 1165
WRONG_COLUMN_NAME: Final = 1166
WRONG_KEY_COLUMN: Final = 1167
WRONG_MRG_TABLE: Final = 1168
DUP_UNIQUE: Final = 1169
BLOB_KEY_WITHOUT_LENGTH: Final = 1170
PRIMARY_CANT_HAVE_NULL: Final = 1171
TOO_MANY_ROWS: Final = 1172
REQUIRES_PRIMARY_KEY: Final = 1173
NO_RAID_COMPILED: Final = 1174
UPDATE_WITHOUT_KEY_IN_SAFE_MODE: Final = 1175
KEY_DOES_NOT_EXITS: Final = 1176
CHECK_NO_SUCH_TABLE: Final = 1177
CHECK_NOT_IMPLEMENTED: Final = 1178
CANT_DO_THIS_DURING_AN_TRANSACTION: Final = 1179
ERROR_DURING_COMMIT: Final = 1180
ERROR_DURING_ROLLBACK: Final = 1181
ERROR_DURING_FLUSH_LOGS: Final = 1182
ERROR_DURING_CHECKPOINT: Final = 1183
NEW_ABORTING_CONNECTION: Final = 1184
DUMP_NOT_IMPLEMENTED: Final = 1185
FLUSH_MASTER_BINLOG_CLOSED: Final = 1186
INDEX_REBUILD: Final = 1187
MASTER: Final = 1188
MASTER_NET_READ: Final = 1189
MASTER_NET_WRITE: Final = 1190
FT_MATCHING_KEY_NOT_FOUND: Final = 1191
LOCK_OR_ACTIVE_TRANSACTION: Final = 1192
UNKNOWN_SYSTEM_VARIABLE: Final = 1193
CRASHED_ON_USAGE: Final = 1194
CRASHED_ON_REPAIR: Final = 1195
WARNING_NOT_COMPLETE_ROLLBACK: Final = 1196
TRANS_CACHE_FULL: Final = 1197
SLAVE_MUST_STOP: Final = 1198
SLAVE_NOT_RUNNING: Final = 1199
BAD_SLAVE: Final = 1200
MASTER_INFO: Final = 1201
SLAVE_THREAD: Final = 1202
TOO_MANY_USER_CONNECTIONS: Final = 1203
SET_CONSTANTS_ONLY: Final = 1204
LOCK_WAIT_TIMEOUT: Final = 1205
LOCK_TABLE_FULL: Final = 1206
READ_ONLY_TRANSACTION: Final = 1207
DROP_DB_WITH_READ_LOCK: Final = 1208
CREATE_DB_WITH_READ_LOCK: Final = 1209
WRONG_ARGUMENTS: Final = 1210
NO_PERMISSION_TO_CREATE_USER: Final = 1211
UNION_TABLES_IN_DIFFERENT_DIR: Final = 1212
LOCK_DEADLOCK: Final = 1213
TABLE_CANT_HANDLE_FT: Final = 1214
CANNOT_ADD_FOREIGN: Final = 1215
NO_REFERENCED_ROW: Final = 1216
ROW_IS_REFERENCED: Final = 1217
CONNECT_TO_MASTER: Final = 1218
QUERY_ON_MASTER: Final = 1219
ERROR_WHEN_EXECUTING_COMMAND: Final = 1220
WRONG_USAGE: Final = 1221
WRONG_NUMBER_OF_COLUMNS_IN_SELECT: Final = 1222
CANT_UPDATE_WITH_READLOCK: Final = 1223
MIXING_NOT_ALLOWED: Final = 1224
DUP_ARGUMENT: Final = 1225
USER_LIMIT_REACHED: Final = 1226
SPECIFIC_ACCESS_DENIED_ERROR: Final = 1227
LOCAL_VARIABLE: Final = 1228
GLOBAL_VARIABLE: Final = 1229
NO_DEFAULT: Final = 1230
WRONG_VALUE_FOR_VAR: Final = 1231
WRONG_TYPE_FOR_VAR: Final = 1232
VAR_CANT_BE_READ: Final = 1233
CANT_USE_OPTION_HERE: Final = 1234
NOT_SUPPORTED_YET: Final = 1235
MASTER_FATAL_ERROR_READING_BINLOG: Final = 1236
SLAVE_IGNORED_TABLE: Final = 1237
INCORRECT_GLOBAL_LOCAL_VAR: Final = 1238
WRONG_FK_DEF: Final = 1239
KEY_REF_DO_NOT_MATCH_TABLE_REF: Final = 1240
OPERAND_COLUMNS: Final = 1241
SUBQUERY_NO_1_ROW: Final = 1242
UNKNOWN_STMT_HANDLER: Final = 1243
CORRUPT_HELP_DB: Final = 1244
CYCLIC_REFERENCE: Final = 1245
AUTO_CONVERT: Final = 1246
ILLEGAL_REFERENCE: Final = 1247
DERIVED_MUST_HAVE_ALIAS: Final = 1248
SELECT_REDUCED: Final = 1249
TABLENAME_NOT_ALLOWED_HERE: Final = 1250
NOT_SUPPORTED_AUTH_MODE: Final = 1251
SPATIAL_CANT_HAVE_NULL: Final = 1252
COLLATION_CHARSET_MISMATCH: Final = 1253
SLAVE_WAS_RUNNING: Final = 1254
SLAVE_WAS_NOT_RUNNING: Final = 1255
TOO_BIG_FOR_UNCOMPRESS: Final = 1256
ZLIB_Z_MEM_ERROR: Final = 1257
ZLIB_Z_BUF_ERROR: Final = 1258
ZLIB_Z_DATA_ERROR: Final = 1259
CUT_VALUE_GROUP_CONCAT: Final = 1260
WARN_TOO_FEW_RECORDS: Final = 1261
WARN_TOO_MANY_RECORDS: Final = 1262
WARN_NULL_TO_NOTNULL: Final = 1263
WARN_DATA_OUT_OF_RANGE: Final = 1264
WARN_DATA_TRUNCATED: Final = 1265
WARN_USING_OTHER_HANDLER: Final = 1266
CANT_AGGREGATE_2COLLATIONS: Final = 1267
DROP_USER: Final = 1268
REVOKE_GRANTS: Final = 1269
CANT_AGGREGATE_3COLLATIONS: Final = 1270
CANT_AGGREGATE_NCOLLATIONS: Final = 1271
VARIABLE_IS_NOT_STRUCT: Final = 1272
UNKNOWN_COLLATION: Final = 1273
SLAVE_IGNORED_SSL_PARAMS: Final = 1274
SERVER_IS_IN_SECURE_AUTH_MODE: Final = 1275
WARN_FIELD_RESOLVED: Final = 1276
BAD_SLAVE_UNTIL_COND: Final = 1277
MISSING_SKIP_SLAVE: Final = 1278
UNTIL_COND_IGNORED: Final = 1279
WRONG_NAME_FOR_INDEX: Final = 1280
WRONG_NAME_FOR_CATALOG: Final = 1281
WARN_QC_RESIZE: Final = 1282
BAD_FT_COLUMN: Final = 1283
UNKNOWN_KEY_CACHE: Final = 1284
WARN_HOSTNAME_WONT_WORK: Final = 1285
UNKNOWN_STORAGE_ENGINE: Final = 1286
WARN_DEPRECATED_SYNTAX: Final = 1287
NON_UPDATABLE_TABLE: Final = 1288
FEATURE_DISABLED: Final = 1289
OPTION_PREVENTS_STATEMENT: Final = 1290
DUPLICATED_VALUE_IN_TYPE: Final = 1291
TRUNCATED_WRONG_VALUE: Final = 1292
TOO_MUCH_AUTO_TIMESTAMP_COLS: Final = 1293
INVALID_ON_UPDATE: Final = 1294
UNSUPPORTED_PS: Final = 1295
GET_ERRMSG: Final = 1296
GET_TEMPORARY_ERRMSG: Final = 1297
UNKNOWN_TIME_ZONE: Final = 1298
WARN_INVALID_TIMESTAMP: Final = 1299
INVALID_CHARACTER_STRING: Final = 1300
WARN_ALLOWED_PACKET_OVERFLOWED: Final = 1301
CONFLICTING_DECLARATIONS: Final = 1302
SP_NO_RECURSIVE_CREATE: Final = 1303
SP_ALREADY_EXISTS: Final = 1304
SP_DOES_NOT_EXIST: Final = 1305
SP_DROP_FAILED: Final = 1306
SP_STORE_FAILED: Final = 1307
SP_LILABEL_MISMATCH: Final = 1308
SP_LABEL_REDEFINE: Final = 1309
SP_LABEL_MISMATCH: Final = 1310
SP_UNINIT_VAR: Final = 1311
SP_BADSELECT: Final = 1312
SP_BADRETURN: Final = 1313
SP_BADSTATEMENT: Final = 1314
UPDATE_LOG_DEPRECATED_IGNORED: Final = 1315
UPDATE_LOG_DEPRECATED_TRANSLATED: Final = 1316
QUERY_INTERRUPTED: Final = 1317
SP_WRONG_NO_OF_ARGS: Final = 1318
SP_COND_MISMATCH: Final = 1319
SP_NORETURN: Final = 1320
SP_NORETURNEND: Final = 1321
SP_BAD_CURSOR_QUERY: Final = 1322
SP_BAD_CURSOR_SELECT: Final = 1323
SP_CURSOR_MISMATCH: Final = 1324
SP_CURSOR_ALREADY_OPEN: Final = 1325
SP_CURSOR_NOT_OPEN: Final = 1326
SP_UNDECLARED_VAR: Final = 1327
SP_WRONG_NO_OF_FETCH_ARGS: Final = 1328
SP_FETCH_NO_DATA: Final = 1329
SP_DUP_PARAM: Final = 1330
SP_DUP_VAR: Final = 1331
SP_DUP_COND: Final = 1332
SP_DUP_CURS: Final = 1333
SP_CANT_ALTER: Final = 1334
SP_SUBSELECT_NYI: Final = 1335
STMT_NOT_ALLOWED_IN_SF_OR_TRG: Final = 1336
SP_VARCOND_AFTER_CURSHNDLR: Final = 1337
SP_CURSOR_AFTER_HANDLER: Final = 1338
SP_CASE_NOT_FOUND: Final = 1339
FPARSER_TOO_BIG_FILE: Final = 1340
FPARSER_BAD_HEADER: Final = 1341
FPARSER_EOF_IN_COMMENT: Final = 1342
FPARSER_ERROR_IN_PARAMETER: Final = 1343
FPARSER_EOF_IN_UNKNOWN_PARAMETER: Final = 1344
VIEW_NO_EXPLAIN: Final = 1345
FRM_UNKNOWN_TYPE: Final = 1346
WRONG_OBJECT: Final = 1347
NONUPDATEABLE_COLUMN: Final = 1348
VIEW_SELECT_DERIVED: Final = 1349
VIEW_SELECT_CLAUSE: Final = 1350
VIEW_SELECT_VARIABLE: Final = 1351
VIEW_SELECT_TMPTABLE: Final = 1352
VIEW_WRONG_LIST: Final = 1353
WARN_VIEW_MERGE: Final = 1354
WARN_VIEW_WITHOUT_KEY: Final = 1355
VIEW_INVALID: Final = 1356
SP_NO_DROP_SP: Final = 1357
SP_GOTO_IN_HNDLR: Final = 1358
TRG_ALREADY_EXISTS: Final = 1359
TRG_DOES_NOT_EXIST: Final = 1360
TRG_ON_VIEW_OR_TEMP_TABLE: Final = 1361
TRG_CANT_CHANGE_ROW: Final = 1362
TRG_NO_SUCH_ROW_IN_TRG: Final = 1363
NO_DEFAULT_FOR_FIELD: Final = 1364
DIVISION_BY_ZERO: Final = 1365
TRUNCATED_WRONG_VALUE_FOR_FIELD: Final = 1366
ILLEGAL_VALUE_FOR_TYPE: Final = 1367
VIEW_NONUPD_CHECK: Final = 1368
VIEW_CHECK_FAILED: Final = 1369
PROCACCESS_DENIED_ERROR: Final = 1370
RELAY_LOG_FAIL: Final = 1371
PASSWD_LENGTH: Final = 1372
UNKNOWN_TARGET_BINLOG: Final = 1373
IO_ERR_LOG_INDEX_READ: Final = 1374
BINLOG_PURGE_PROHIBITED: Final = 1375
FSEEK_FAIL: Final = 1376
BINLOG_PURGE_FATAL_ERR: Final = 1377
LOG_IN_USE: Final = 1378
LOG_PURGE_UNKNOWN_ERR: Final = 1379
RELAY_LOG_INIT: Final = 1380
NO_BINARY_LOGGING: Final = 1381
RESERVED_SYNTAX: Final = 1382
WSAS_FAILED: Final = 1383
DIFF_GROUPS_PROC: Final = 1384
NO_GROUP_FOR_PROC: Final = 1385
ORDER_WITH_PROC: Final = 1386
LOGGING_PROHIBIT_CHANGING_OF: Final = 1387
NO_FILE_MAPPING: Final = 1388
WRONG_MAGIC: Final = 1389
PS_MANY_PARAM: Final = 1390
KEY_PART_0: Final = 1391
VIEW_CHECKSUM: Final = 1392
VIEW_MULTIUPDATE: Final = 1393
VIEW_NO_INSERT_FIELD_LIST: Final = 1394
VIEW_DELETE_MERGE_VIEW: Final = 1395
CANNOT_USER: Final = 1396
XAER_NOTA: Final = 1397
XAER_INVAL: Final = 1398
XAER_RMFAIL: Final = 1399
XAER_OUTSIDE: Final = 1400
XAER_RMERR: Final = 1401
XA_RBROLLBACK: Final = 1402
NONEXISTING_PROC_GRANT: Final = 1403
PROC_AUTO_GRANT_FAIL: Final = 1404
PROC_AUTO_REVOKE_FAIL: Final = 1405
DATA_TOO_LONG: Final = 1406
SP_BAD_SQLSTATE: Final = 1407
STARTUP: Final = 1408
LOAD_FROM_FIXED_SIZE_ROWS_TO_VAR: Final = 1409
CANT_CREATE_USER_WITH_GRANT: Final = 1410
WRONG_VALUE_FOR_TYPE: Final = 1411
TABLE_DEF_CHANGED: Final = 1412
SP_DUP_HANDLER: Final = 1413
SP_NOT_VAR_ARG: Final = 1414
SP_NO_RETSET: Final = 1415
CANT_CREATE_GEOMETRY_OBJECT: Final = 1416
FAILED_ROUTINE_BREAK_BINLOG: Final = 1417
BINLOG_UNSAFE_ROUTINE: Final = 1418
BINLOG_CREATE_ROUTINE_NEED_SUPER: Final = 1419
EXEC_STMT_WITH_OPEN_CURSOR: Final = 1420
STMT_HAS_NO_OPEN_CURSOR: Final = 1421
COMMIT_NOT_ALLOWED_IN_SF_OR_TRG: Final = 1422
NO_DEFAULT_FOR_VIEW_FIELD: Final = 1423
SP_NO_RECURSION: Final = 1424
TOO_BIG_SCALE: Final = 1425
TOO_BIG_PRECISION: Final = 1426
M_BIGGER_THAN_D: Final = 1427
WRONG_LOCK_OF_SYSTEM_TABLE: Final = 1428
CONNECT_TO_FOREIGN_DATA_SOURCE: Final = 1429
QUERY_ON_FOREIGN_DATA_SOURCE: Final = 1430
FOREIGN_DATA_SOURCE_DOESNT_EXIST: Final = 1431
FOREIGN_DATA_STRING_INVALID_CANT_CREATE: Final = 1432
FOREIGN_DATA_STRING_INVALID: Final = 1433
CANT_CREATE_FEDERATED_TABLE: Final = 1434
TRG_IN_WRONG_SCHEMA: Final = 1435
STACK_OVERRUN_NEED_MORE: Final = 1436
TOO_LONG_BODY: Final = 1437
WARN_CANT_DROP_DEFAULT_KEYCACHE: Final = 1438
TOO_BIG_DISPLAYWIDTH: Final = 1439
XAER_DUPID: Final = 1440
DATETIME_FUNCTION_OVERFLOW: Final = 1441
CANT_UPDATE_USED_TABLE_IN_SF_OR_TRG: Final = 1442
VIEW_PREVENT_UPDATE: Final = 1443
PS_NO_RECURSION: Final = 1444
SP_CANT_SET_AUTOCOMMIT: Final = 1445
MALFORMED_DEFINER: Final = 1446
VIEW_FRM_NO_USER: Final = 1447
VIEW_OTHER_USER: Final = 1448
NO_SUCH_USER: Final = 1449
FORBID_SCHEMA_CHANGE: Final = 1450
ROW_IS_REFERENCED_2: Final = 1451
NO_REFERENCED_ROW_2: Final = 1452
SP_BAD_VAR_SHADOW: Final = 1453
TRG_NO_DEFINER: Final = 1454
OLD_FILE_FORMAT: Final = 1455
SP_RECURSION_LIMIT: Final = 1456
SP_PROC_TABLE_CORRUPT: Final = 1457
SP_WRONG_NAME: Final = 1458
TABLE_NEEDS_UPGRADE: Final = 1459
SP_NO_AGGREGATE: Final = 1460
MAX_PREPARED_STMT_COUNT_REACHED: Final = 1461
VIEW_RECURSIVE: Final = 1462
NON_GROUPING_FIELD_USED: Final = 1463
TABLE_CANT_HANDLE_SPKEYS: Final = 1464
NO_TRIGGERS_ON_SYSTEM_SCHEMA: Final = 1465
USERNAME: Final = 1466
HOSTNAME: Final = 1467
WRONG_STRING_LENGTH: Final = 1468
ERROR_LAST: Final = 1468
STATEMENT_TIMEOUT: Final = 1969
QUERY_TIMEOUT: Final = 3024
CONSTRAINT_FAILED: Final = 4025



from types import FrameType
from typing import ClassVar

from gunicorn.app.base import BaseApplication
from gunicorn.config import Config
from gunicorn.glogging import Logger as GLogger
from gunicorn.sock import BaseSocket
from gunicorn.workers.base import Worker

from ._types import _AddressType
from .pidfile import Pidfile

class Arbiter:
    WORKER_BOOT_ERROR: ClassVar[int]
    APP_LOAD_ERROR: ClassVar[int]
    START_CTX: ClassVar[dict[int | str, str | list[str]]]
    LISTENERS: ClassVar[list[BaseSocket]]
    WORKERS: ClassVar[dict[int, Worker]]
    PIPE: ClassVar[list[int]]
    SIG_QUEUE: ClassVar[list[int]]
    SIGNALS: ClassVar[list[int]]
    SIG_NAMES: ClassVar[dict[int, str]]
    log: GLogger | None
    pidfile: Pidfile | None
    systemd: bool
    worker_age: int
    reexec_pid: int
    master_pid: int
    master_name: str
    pid: int
    app: BaseApplication
    cfg: Config
    worker_class: type[Worker]
    address: list[_AddressType]
    timeout: int
    proc_name: str
    num_workers: int

    def __init__(self, app: BaseApplication) -> None: ...
    def setup(self, app: BaseApplication) -> None: ...
    def start(self) -> None: ...
    def init_signals(self) -> None: ...
    def signal(self, sig: int, frame: FrameType | None) -> None: ...
    def run(self) -> None: ...
    def handle_chld(self, sig: int, frame: FrameType | None) -> None: ...
    def handle_hup(self) -> None: ...
    def handle_term(self) -> None: ...
    def handle_int(self) -> None: ...
    def handle_quit(self) -> None: ...
    def handle_ttin(self) -> None: ...
    def handle_ttou(self) -> None: ...
    def handle_usr1(self) -> None: ...
    def handle_usr2(self) -> None: ...
    def handle_winch(self) -> None: ...
    def maybe_promote_master(self) -> None: ...
    def wakeup(self) -> None: ...
    def halt(self, reason: str | None = None, exit_status: int = 0) -> None: ...
    def sleep(self) -> None: ...
    def stop(self, graceful: bool = True) -> None: ...
    def reexec(self) -> None: ...
    def reload(self) -> None: ...
    def murder_workers(self) -> None: ...
    def reap_workers(self) -> None: ...
    def manage_workers(self) -> None: ...
    def spawn_worker(self) -> int: ...
    def spawn_workers(self) -> None: ...
    def kill_workers(self, sig: int) -> None: ...
    def kill_worker(self, pid: int, sig: int) -> None: ...



from collections.abc import Sequence
from typing import Final

from Xlib.display import Display
from Xlib.protocol import request, rq
from Xlib.protocol.structs import _Rectangle4IntSequence
from Xlib.xobject import drawable, resource

extname: Final = "SHAPE"
OP = rq.Card8

class SO:
    Set: int
    Union: int
    Intersect: int
    Subtract: int
    Invert: int

class SK:
    Bounding: int
    Clip: int
    Input: int

class KIND(rq.Set):
    def __init__(self, name: str) -> None: ...

class NotifyEventData(rq.Event): ...
class QueryVersion(rq.ReplyRequest): ...
class Rectangles(rq.Request): ...
class Mask(rq.Request): ...
class Combine(rq.Request): ...
class Offset(rq.Request): ...
class QueryExtents(rq.ReplyRequest): ...
class SelectInput(rq.Request): ...
class InputSelected(rq.ReplyRequest): ...
class GetRectangles(rq.ReplyRequest): ...

class Event:
    Notify: int

def combine(
    self: drawable.Window, operation: int, destination_kind: int, source_kind: int, x_offset: int, y_offset: int
) -> None: ...
def get_rectangles(self: drawable.Window, source_kind: int) -> GetRectangles: ...
def input_selected(self: drawable.Window) -> InputSelected: ...
def mask(
    self: drawable.Window, operation: int, destination_kind: int, x_offset: int, y_offset: int, source_bitmap: int
) -> None: ...
def offset(self: drawable.Window, destination_kind: int, x_offset: int, y_offset: int) -> None: ...
def query_extents(self: drawable.Window) -> QueryExtents: ...
def query_version(self: Display | resource.Resource) -> QueryVersion: ...
def rectangles(
    self: drawable.Window,
    operation: int,
    destination_kind: int,
    ordering: int,
    x_offset: int,
    y_offset: int,
    rectangles: Sequence[_Rectangle4IntSequence],
) -> None: ...
def select_input(self: drawable.Window, enable: int) -> None: ...
def init(disp: Display, info: request.QueryExtension) -> None: ...



import enum
import html.parser
import json
import netrc
import optparse
import subprocess
import sys
import types
from _typeshed import (
    ExcInfo,
    FileDescriptorLike,
    FileDescriptorOrPath,
    OpenBinaryMode,
    OpenTextMode,
    ReadableBuffer,
    StrOrBytesPath,
    Unused,
)
from collections import deque
from collections.abc import Callable, Collection, Hashable, Iterable, Iterator, Mapping, Sequence
from datetime import date, datetime, timedelta
from functools import cache
from optparse import Values
from os import PathLike
from re import Pattern
from typing import IO, Any, AnyStr, BinaryIO, Final, Generic, Literal, NamedTuple, TextIO, TypeVar, overload
from typing_extensions import Self, TypeAlias
from urllib.parse import _QueryType, _QuoteVia
from xml.etree import ElementTree as ET

from yt_dlp.networking import Response

from .. import _Params
from ..extractor.common import InfoExtractor, _InfoDict
from ..globals import WINDOWS_VT_MODE as WINDOWS_VT_MODE
from ..options import _YoutubeDLOptionParser
from ..YoutubeDL import YoutubeDL

_T = TypeVar("_T")

class NO_DEFAULT: ...

def IDENTITY(x: _T) -> _T: ...

ENGLISH_MONTH_NAMES: Final[Sequence[str]]
MONTH_NAMES: Final[Mapping[str, Sequence[str]]]
TIMEZONE_NAMES: Final[Mapping[str, str]]
ACCENT_CHARS: Final[Mapping[str, str]]
DATE_FORMATS: Final[Sequence[str]]
DATE_FORMATS_DAY_FIRST: Final[Sequence[str]]
DATE_FORMATS_MONTH_FIRST: Final[Sequence[str]]
PACKED_CODES_RE: Final[str]
JSON_LD_RE: Final[str]
NUMBER_RE: Final[str]

@cache
def preferredencoding() -> str: ...
def write_json_file(obj: Any, fn: str) -> None: ...  # obj passed to json.dump().
def partial_application(func: Callable[..., Any]) -> Callable[..., Any]: ...
def find_xpath_attr(node: ET.ElementTree, xpath: str, key: str, val: str | None = None) -> ET.Element | None: ...
def xpath_with_ns(path: str, ns_map: Mapping[str, str]) -> str: ...
def xpath_element(
    node: ET.ElementTree, xpath: str, name: str | None = None, fatal: bool = False, default: ET.Element | type[NO_DEFAULT] = ...
) -> ET.Element | None: ...
def xpath_text(
    node: ET.ElementTree, xpath: str, name: str | None = None, fatal: bool = False, default: str | type[NO_DEFAULT] = ...
) -> str | None: ...
def xpath_attr(
    node: ET.ElementTree,
    xpath: str,
    key: str,
    name: str | None = None,
    fatal: bool = False,
    default: str | type[NO_DEFAULT] = ...,
) -> str | None: ...
def get_element_by_id(id: str, html: str, *, tag: str, escape_value: bool = True) -> str | None: ...
def get_element_html_by_id(id: str, html: str, *, tag: str, escape_value: bool = True) -> str | None: ...
def get_element_by_class(class_name: str, html: str) -> str: ...
def get_element_html_by_class(class_name: str, html: str) -> str: ...
def get_element_by_attribute(attribute: str, value: str, html: str, *, tag: str, escape_value: bool = True) -> str: ...
def get_element_html_by_attribute(attribute: str, value: str, html: str, *, tag: str, escape_value: bool = True) -> list[str]: ...
def get_elements_by_class(class_name: str, html: str, **kargs: Unused) -> list[str]: ...
def get_elements_html_by_class(class_name: str, html: str) -> list[str]: ...
def get_elements_by_attribute(attribute: str, value: str, html: str, *, tag: str, escape_value: bool = True) -> list[str]: ...
def get_elements_html_by_attribute(
    attribute: str, value: str, html: str, *, tag: str = "[\\w:.-]+", escape_value: bool = True
) -> list[str]: ...
def get_elements_text_and_html_by_attribute(
    attribute: str, value: str, html: str, *, tag: str = "[\\w:.-]+", escape_value: bool = True
) -> Iterator[str]: ...

class HTMLBreakOnClosingTagParser(html.parser.HTMLParser):
    class HTMLBreakOnClosingTagException(Exception): ...
    tagstack: deque[Any]
    def __init__(self) -> None: ...
    def __enter__(self) -> Self: ...
    def __exit__(self, *_: Unused) -> None: ...
    def close(self) -> None: ...
    def handle_starttag(self, tag: str, _: Unused) -> None: ...
    def handle_endtag(self, tag: str) -> None: ...

def get_element_text_and_html_by_tag(tag: str, html: str) -> str: ...

class HTMLAttributeParser(html.parser.HTMLParser):
    attrs: dict[str, str | None]
    def __init__(self) -> None: ...
    def handle_starttag(self, tag: str, attrs: list[tuple[str, str | None]]) -> None: ...

class HTMLListAttrsParser(html.parser.HTMLParser):
    items: list[dict[str, str | None]]
    def __init__(self) -> None: ...
    def handle_starttag(self, tag: str, attrs: list[tuple[str, str | None]]) -> None: ...
    def handle_endtag(self, tag: str) -> None: ...

def extract_attributes(html_element: str) -> dict[str, str]: ...
def parse_list(webpage: str) -> list[dict[str, str | None]]: ...
def clean_html(html: str | None) -> str | None: ...

class LenientJSONDecoder(json.JSONDecoder):
    def __init__(
        self,
        *args: Unused,
        transform_source: Callable[[str], str] | None = None,
        ignore_extra: bool = False,
        close_objects: int = 0,
        object_hook: Callable[[dict[str, Any]], Any] | None = None,
        parse_float: Callable[[str], Any] | None = None,
        parse_int: Callable[[str], Any] | None = None,
        parse_constant: Callable[[str], Any] | None = None,
        strict: bool = True,
        object_pairs_hook: Callable[[list[tuple[str, Any]]], Any] | None = None,
    ) -> None: ...
    def decode(self, s: str) -> Any: ...  # type: ignore[override]

@overload
def sanitize_open(filename: FileDescriptorOrPath, open_mode: OpenBinaryMode) -> BinaryIO: ...
@overload
def sanitize_open(filename: FileDescriptorOrPath, open_mode: OpenTextMode) -> TextIO: ...
def timeconvert(timestr: str) -> str: ...
def sanitize_filename(s: str, restricted: bool = False, is_id: bool | type[NO_DEFAULT] = ...) -> str: ...
def sanitize_path(s: str, force: bool = False) -> str: ...
def sanitize_url(url: str, *, scheme: str = "http") -> str: ...
def extract_basic_auth(url: str) -> tuple[str, str | None]: ...
def expand_path(s: str) -> str: ...
def orderedSet(iterable: Iterable[_T], *, lazy: bool = False) -> Iterator[_T]: ...
def unescapeHTML(s: str | None) -> str | None: ...
def escapeHTML(text: str) -> str: ...

class netrc_from_content(netrc.netrc):
    def __init__(self, content: str) -> None: ...

def encodeArgument(s: str) -> str: ...

class _timetuple(NamedTuple):
    hours: tuple[int, int]
    minutes: tuple[int, int]
    seconds: tuple[int, int]
    milliseconds: tuple[int, int]

def timetuple_from_msec(msec: int) -> _timetuple: ...
def formatSeconds(secs: int, delim: str = ":", msec: bool = False) -> str: ...
def bug_reports_message(before: str = ";") -> None: ...

class YoutubeDLError(Exception):
    msg: str | None
    def __init__(self, msg: str | None = None) -> None: ...

class ExtractorError(YoutubeDLError):
    orig_msg: Any
    traceback: types.TracebackType | None
    expected: Any
    cause: Exception | str | None
    video_id: str
    ie: InfoExtractor
    exc_info: ExcInfo
    def __init__(
        self,
        msg: str,
        tb: types.TracebackType | None = None,
        expected: bool = False,
        cause: Exception | str | None = None,
        video_id: str | None = None,
        ie: InfoExtractor | None = None,
    ) -> None: ...
    def format_traceback(self) -> str: ...
    msg: str | None
    args: tuple[Any, ...]
    def __setattr__(self, name: str, value: Any) -> None: ...

class UnsupportedError(ExtractorError):
    url: str
    def __init__(self, url: str) -> None: ...

class RegexNotFoundError(ExtractorError): ...

class GeoRestrictedError(ExtractorError):
    countries: str | None
    def __init__(
        self,
        msg: str,
        countries: str | None = None,
        *,
        tb: types.TracebackType | None = None,
        expected: bool = False,
        cause: Exception | str | None = None,
        video_id: str | None = None,
        ie: InfoExtractor | None = None,
    ) -> None: ...

class UserNotLive(ExtractorError):
    def __init__(
        self,
        msg: str | None = None,
        *,
        tb: types.TracebackType | None = None,
        expected: bool = False,
        cause: Exception | str | None = None,
        video_id: str | None = None,
        ie: InfoExtractor | None = None,
    ) -> None: ...

class DownloadError(YoutubeDLError):
    exc_info: ExcInfo
    def __init__(self, msg: str, exc_info: ExcInfo | None = None) -> None: ...

class EntryNotInPlaylist(YoutubeDLError):
    msg: str

class SameFileError(YoutubeDLError):
    msg: str
    def __init__(self, filename: str | None = None) -> None: ...

class PostProcessingError(YoutubeDLError): ...

class DownloadCancelled(YoutubeDLError):
    msg: str

class ExistingVideoReached(DownloadCancelled):
    msg: str

class RejectedVideoReached(DownloadCancelled):
    msg: str

class MaxDownloadsReached(DownloadCancelled):
    msg: str

class ReExtractInfo(YoutubeDLError):
    expected: bool
    def __init__(self, msg: str, expected: bool = False) -> None: ...

class ThrottledDownload(ReExtractInfo):
    msg: str
    def __init__(self) -> None: ...

class UnavailableVideoError(YoutubeDLError):
    msg: str
    def __init__(self, err: str | None = None) -> None: ...

class ContentTooShortError(YoutubeDLError):
    downloaded: int
    expected: int
    def __init__(self, downloaded: int, expected: int) -> None: ...

class XAttrMetadataError(YoutubeDLError):
    code: str | None
    msg: str | None
    reason: str
    def __init__(self, code: str | None = None, msg: str = "Unknown error") -> None: ...

class XAttrUnavailableError(YoutubeDLError): ...

def is_path_like(f: Any) -> bool: ...  # Type checker.
def extract_timezone(date_str: str, default: Any = None) -> tuple[timedelta, str]: ...  # Any or type[NO_DEFAULT]
def parse_iso8601(date_str: str, delimiter: str = "T", timezone: type[NO_DEFAULT] | Any | None = None) -> int: ...
def date_formats(day_first: bool = True) -> list[str]: ...
def unified_strdate(date_str: str, day_first: bool = True) -> str: ...
def unified_timestamp(date_str: str, day_first: bool = True) -> int: ...
def determine_ext(url: str, default_ext: str = "unknown_video") -> str: ...
def subtitles_filename(filename: str, sub_lang: str, sub_format: str, expected_real_ext: str | None = None) -> str: ...
def datetime_from_str(date_str: str, precision: str = "auto", format: str = "%Y%m%d") -> datetime: ...
def date_from_str(date_str: str, format: str = "%Y%m%d", strict: bool = False) -> date: ...
def datetime_add_months(dt_: datetime, months: int) -> datetime: ...
def datetime_round(dt_: datetime, precision: str = "day") -> datetime: ...
def hyphenate_date(date_str: str) -> str: ...

class DateRange:
    start: date
    end: date
    def __init__(self, start: date | None = None, end: date | None = None) -> None: ...
    @classmethod
    def day(cls, day: date) -> Self: ...
    def __contains__(self, date: date) -> bool: ...
    def __eq__(self, other: object) -> bool: ...

def system_identifier() -> str: ...
def get_windows_version() -> tuple[str, ...]: ...
def write_string(s: str, out: TextIO | None = None, encoding: str | None = None) -> None: ...
def deprecation_warning(
    msg: str, *, printer: Callable[..., Any] | None = None, stacklevel: int = 0, **kwargs: Any  # kwargs are passed to printer.
) -> None: ...

class LockingUnsupportedError(OSError):
    msg: str
    def __init__(self) -> None: ...

class locked_file:
    locked: bool
    f: TextIO
    def __init__(
        self, filename: AnyStr, mode: OpenTextMode | OpenBinaryMode, block: bool = True, encoding: str | None = None
    ) -> None: ...
    def __enter__(self) -> Self: ...
    def unlock(self) -> None: ...
    def __exit__(self, *_: Unused) -> None: ...
    open = __enter__
    close = __exit__
    def __getattr__(self, attr: str) -> Any: ...
    def __iter__(self) -> str: ...

def get_filesystem_encoding() -> str: ...
def shell_quote(args: str | Collection[str], *, shell: bool = False) -> str: ...
def smuggle_url(url: str, data: Any) -> str: ...  # data may be anything JSON serializable.

# default is simply returned if #__youtubedl_smuggle is present.
def unsmuggle_url(smug_url: str, default: Any | None = None) -> tuple[str, Any]: ...
def format_decimal_suffix(num: float, fmt: str = "%d%s", *, factor: int = 1000) -> str: ...
def format_bytes(bytes: int) -> str: ...
def lookup_unit_table(unit_table: Mapping[str, int], s: str, strict: bool = False) -> float: ...
def parse_bytes(s: str) -> int: ...
def parse_filesize(s: str | None) -> int | None: ...
def parse_count(s: str | None) -> str | None: ...
def parse_resolution(s: str, *, lenient: bool = False) -> dict[str, int]: ...
def parse_bitrate(s: str) -> int: ...
def month_by_name(name: str, lang: str = "en") -> str | None: ...
def month_by_abbreviation(abbrev: str) -> str | None: ...
def fix_xml_ampersands(xml_str: str) -> str: ...
def setproctitle(title: str) -> None: ...
def remove_start(s: str, start: str) -> str: ...
def remove_end(s: str, end: str) -> str: ...
def remove_quotes(s: str) -> str: ...
def get_domain(url: str) -> str | None: ...
def url_basename(url: str) -> str: ...
def base_url(url: str) -> str: ...
def urljoin(base: str, path: str) -> str: ...
def int_or_none(
    v: Any, scale: int = 1, default: int | None = None, get_attr: str | None = None, invscale: int = 1, base: int | None = None
) -> int | None: ...
def str_or_none(v: Any, default: str | None = None) -> str: ...
def str_to_int(int_str: str) -> int: ...
def float_or_none(v: Any, scale: int = 1, invscale: int = 1, default: float | None = None) -> float | None: ...
def bool_or_none(v: Any, default: bool | None = None) -> bool | None: ...
def strip_or_none(v: Any, default: str | None = None) -> str | None: ...
def url_or_none(url: Any) -> str | None: ...
def strftime_or_none(timestamp: int, date_format: str = "%Y%m%d", default: str | None = None) -> str | None: ...
def parse_duration(s: str | None) -> float: ...
def prepend_extension(filename: str, ext: str, expected_real_ext: str | None = None) -> str: ...
def replace_extension(filename: str, ext: str, expected_real_ext: str | None = None) -> str: ...
def check_executable(exe: str, args: Iterable[str] = []) -> str | None: ...
def detect_exe_version(output: str, version_re: str | Pattern[str] | None = None, unrecognized: str = "present") -> str: ...
def get_exe_version(
    exe: str,
    args: Iterable[str] = ["--version"],
    version_re: str | None = None,
    unrecognized: Iterable[str] = ("present", "broken"),
) -> str: ...
def frange(start: int = 0, stop: int | None = None, step: int = 1) -> Iterator[float]: ...

class LazyList(Sequence[_T]):
    def __init__(self, iterable: Iterable[_T], *, reverse: bool = False, _cache: list[Any] | None = None) -> None: ...
    def __iter__(self) -> Iterator[_T]: ...
    def exhaust(self) -> list[_T]: ...
    @overload
    def __getitem__(self, idx: int, /) -> _T: ...
    @overload
    def __getitem__(self, idx: slice, /) -> list[_T]: ...
    def __bool__(self) -> bool: ...
    def __len__(self) -> int: ...
    def __reversed__(self) -> Iterator[_T]: ...
    def __copy__(self) -> Self: ...

class PagedList:
    def __len__(self) -> int: ...
    def __init__(self, pagefunc: Callable[[int], Iterator[Any]], pagesize: int, use_cache: bool = True) -> None: ...
    def getpage(self, pagenum: int) -> list[Any]: ...
    def getslice(self, start: int = 0, end: int | None = None) -> list[Any]: ...
    @overload
    def __getitem__(self, idx: int, /) -> Any: ...
    @overload
    def __getitem__(self, idx: slice, /) -> list[Any]: ...
    def __bool__(self) -> bool: ...

class OnDemandPagedList(PagedList): ...

class InAdvancePagedList(PagedList):
    def __init__(self, pagefunc: Callable[[int], Iterator[Any]], pagecount: int, pagesize: int) -> None: ...

class PlaylistEntries:
    MissingEntry: Any
    is_exhausted: bool
    ydl: YoutubeDL
    is_incomplete: bool
    def __init__(self, ydl: YoutubeDL, info_dict: _InfoDict) -> None: ...
    PLAYLIST_ITEMS_RE: Pattern[str]
    @classmethod
    def parse_playlist_items(cls, string: str) -> slice | int: ...
    def get_requested_items(self) -> Iterator[tuple[int, Any]]: ...
    def get_full_count(self) -> int | None: ...
    def __getitem__(self, idx: int) -> Iterator[tuple[int, Any]]: ...
    def __len__(self) -> int: ...

_K = TypeVar("_K")
_V = TypeVar("_V")

def uppercase_escape(s: str) -> str: ...
def lowercase_escape(s: str) -> str: ...
def parse_qs(
    url: str,
    *,
    keep_blank_values: bool = False,
    strict_parsing: bool = False,
    encoding: str = "utf-8",
    errors: str = "replace",
    max_num_fields: int | None = None,
    separator: str = "&",
) -> dict[AnyStr, list[AnyStr]]: ...
def read_batch_urls(batch_fd: FileDescriptorLike) -> list[str]: ...
def urlencode_postdata(
    query: _QueryType,
    doseq: bool = False,
    safe: str | bytes = "",
    encoding: str | None = None,
    errors: str | None = None,
    quote_via: _QuoteVia = ...,
) -> bytes: ...

# Passes kwargs to NamedTuple._replace().
def update_url(url: str, *, query_update: Mapping[str, str] | None = None, **kwargs: Any) -> str: ...
def update_url_query(url: str, query: Mapping[str, str]) -> str: ...
def multipart_encode(data: Mapping[AnyStr, AnyStr], boundary: str | None = None) -> tuple[bytes, str]: ...
def is_iterable_like(
    x: Any, allowed_types: Collection[type[Any]] = ..., blocked_types: Collection[type[Any]] | type[NO_DEFAULT] = ...
) -> bool: ...
def variadic(x: _T, allowed_types: Collection[type[Any]] | type[NO_DEFAULT] = ...) -> _T | tuple[_T]: ...
def try_call(
    *funcs: Callable[..., _T],
    expected_type: type[_T] | None = None,
    args: Iterable[Any] = [],
    kwargs: Mapping[Hashable, Any] = {},
) -> _T | None: ...
def try_get(src: Any, getter: Callable[..., _T] | Collection[Callable[..., _T]], expected_type: type[_T] | None = None) -> _T: ...
def filter_dict(dct: Mapping[_K, _V], cndn: Callable[[_K, _V], bool] = ...) -> dict[_K, _V]: ...
def merge_dicts(*dicts: Mapping[Hashable, Any]) -> dict[Hashable, Any]: ...
def encode_compat_str(string: str, encoding: str = ..., errors: str = "strict") -> str: ...

US_RATINGS: Final[Mapping[str, int]]
TV_PARENTAL_GUIDELINES: Final[Mapping[str, int]]

def parse_age_limit(s: int) -> int | None: ...
def strip_jsonp(code: str) -> str: ...
def js_to_json(code: str, vars: Mapping[str, Any] = {}, *, strict: bool = False) -> str: ...
def qualities(quality_ids: Sequence[int]) -> Callable[[int], int]: ...

POSTPROCESS_WHEN: Final[tuple[str, ...]]
DEFAULT_OUTTMPL: Final[Mapping[str, str]]
OUTTMPL_TYPES: Final[Mapping[str, str | None]]
STR_FORMAT_RE_TMPL: Final[str]
STR_FORMAT_TYPES: Final[str]

def limit_length(s: str, length: int) -> str: ...
def version_tuple(v: str) -> tuple[int, ...]: ...
def is_outdated_version(version: str, limit: str, assume_new: bool = True) -> bool: ...
def ytdl_is_updateable() -> bool: ...
def args_to_str(args: str | Collection[str]) -> str: ...
def error_to_str(err: BaseException) -> str: ...
def mimetype2ext(mt: str, default: str | type[NO_DEFAULT] = ...) -> str: ...
def ext2mimetype(ext_or_url: str | None) -> str: ...
def parse_codecs(codecs_str: str) -> dict[str, str]: ...
def get_compatible_ext(
    *,
    vcodecs: Collection[str],
    acodecs: Collection[str],
    vexts: Collection[str],
    aexts: Collection[str],
    preferences: Sequence[str] | None = None,
) -> str: ...
def urlhandle_detect_ext(url_handle: Response, default: str | type[NO_DEFAULT] = ...) -> str | None: ...
def encode_data_uri(data: ReadableBuffer, mime_type: str) -> str: ...
def age_restricted(content_limit: int | None, age_limit: int | None) -> bool: ...

BOMS: Final[Collection[tuple[bytes, str]]]

def is_html(first_bytes: bytes) -> bool: ...
def determine_protocol(info_dict: _InfoDict) -> str: ...
def render_table(
    header_row: Iterable[str], data: Iterable[str], delim: bool = False, extra_gap: int = 0, hide_empty: bool = False
) -> str: ...
def match_str(filter_str: str, dct: Mapping[str, Any], incomplete: bool = False) -> bool: ...
def match_filter_func(
    filters: Collection[str] | str, breaking_filters: Collection[str] | str | None = None
) -> Callable[..., str | type[NO_DEFAULT] | None]: ...

class download_range_func:
    def __init__(
        self, chapters: Iterable[str | Pattern[str]], ranges: Iterable[tuple[int, int]], from_info: bool = False
    ) -> None: ...
    def __call__(self, info_dict: _InfoDict, ydl: YoutubeDL) -> Iterator[dict[str, Any]]: ...
    def __eq__(self, other: object) -> bool: ...

def parse_dfxp_time_expr(time_expr: str | None) -> int | None: ...
def srt_subtitles_timecode(seconds: float) -> str: ...
def ass_subtitles_timecode(seconds: float) -> str: ...
def dfxp2srt(dfxp_data: bytes) -> str: ...
def cli_option(params: _Params, command_option: str, param: str, separator: str | None = None) -> Any: ...
def cli_bool_option(
    params: _Params,
    command_option: str,
    param: bool | None,
    true_value: str = "true",
    false_value: str = "false",
    separator: str | None = None,
) -> Any: ...
def cli_valueless_option(params: _Params, command_option: str, param: str, expected_value: bool = True) -> Any: ...
def cli_configuration_args(argdict: dict[str, Any], keys: Iterable[str], default: Any = [], use_compat: bool = True) -> Any: ...

class ISO639Utils:
    @classmethod
    def short2long(cls, code: str) -> str | None: ...
    @classmethod
    def long2short(cls, code: str) -> str | None: ...

class ISO3166Utils:
    @classmethod
    def short2full(cls, code: str) -> str | None: ...

class GeoUtils:
    @classmethod
    def random_ipv4(cls, code_or_block: str) -> str | None: ...

def long_to_bytes(n: int, blocksize: int = 0) -> bytes: ...
def bytes_to_long(s: bytes) -> int: ...
def ohdave_rsa_encrypt(data: ReadableBuffer, exponent: float, modulus: float | None) -> str: ...
def pkcs1pad(data: Sequence[int], length: int) -> list[int]: ...
def encode_base_n(num: int, n: int | None = None, table: str | None = None) -> str: ...
def decode_base_n(string: str, n: int | None = None, table: str | None = None) -> int: ...
def decode_packed_codes(code: str) -> str: ...
def caesar(s: str, alphabet: str, shift: int) -> str: ...
def rot47(s: str) -> str: ...
def parse_m3u8_attributes(attrib: str) -> dict[str, str]: ...
def urshift(val: int, n: int) -> int: ...
def write_xattr(path: FileDescriptorOrPath, key: str, value: str) -> None: ...
def random_birthday(year_field: Hashable, month_field: Hashable, day_field: Hashable) -> dict[Hashable, str]: ...
def find_available_port(interface: str = "") -> Any | None: ...

DOT_URL_LINK_TEMPLATE: Final[str]
DOT_WEBLOC_LINK_TEMPLATE: Final[str]
DOT_DESKTOP_LINK_TEMPLATE: Final[str]
LINK_TEMPLATES: Final[Mapping[str, str]]

def iri_to_uri(iri: str) -> str: ...
def to_high_limit_path(path: PathLike[AnyStr]) -> str: ...
def format_field(
    obj: Mapping[str, Any] | Sequence[Any],
    field: str | Collection[str] | None = None,
    template: str = "%s",
    ignore: type[NO_DEFAULT] | str | Collection[str] = ...,
    default: str = "",
    func: Callable[[Any], Any] = ...,
) -> str: ...
def clean_podcast_url(url: str) -> str: ...
def random_uuidv4() -> str: ...
def make_dir(path: PathLike[AnyStr], to_screen: Callable[[str], Any] | None = None) -> bool: ...
def get_executable_path() -> str: ...
def get_user_config_dirs(package_name: str) -> Iterator[str]: ...
def get_system_config_dirs(package_name: str) -> Iterator[str]: ...
def time_seconds(**kwargs: float) -> int: ...
def jwt_encode(
    payload_data: Any, key: str, *, alg: Literal["HS256"] = "HS256", headers: Mapping[str, Any] | None = None
) -> str: ...  # payload_data and headers are passed to json.dumps().
def jwt_decode_hs256(jwt: str) -> Any: ...  # Returns json.loads() output.
def supports_terminal_sequences(stream: IO[Any]) -> bool: ...
def windows_enable_vt_mode() -> None: ...
def remove_terminal_sequences(string: str) -> str: ...
def number_of_digits(number: int) -> int: ...
def join_nonempty(*values: str, delim: str = "-", from_dict: Mapping[str, Any] | None = None) -> str: ...
def scale_thumbnails_to_max_format_width(
    formats: Iterable[Mapping[str, Any]], thumbnails: Iterable[Mapping[str, Any]], url_width_re: str | Pattern[str]
) -> list[dict[str, Any]]: ...
def parse_http_range(range: str | None) -> tuple[int | None, int | None, int | None]: ...
def read_stdin(what: str) -> TextIO | Any: ...
def determine_file_encoding(data: bytes) -> tuple[str | None, int]: ...

class Config:
    own_args: Sequence[str] | None
    parsed_args: tuple[Values, list[str]] | None
    filename: str | None
    def __init__(self, parser: _YoutubeDLOptionParser, label: str | None = None) -> None: ...
    def init(self, args: Sequence[str] | None = None, filename: str | None = None) -> bool: ...
    def load_configs(self) -> bool: ...
    @staticmethod
    def read_file(filename: FileDescriptorOrPath, default: list[str] = []) -> list[str]: ...
    @staticmethod
    def hide_login_info(opts: Iterable[str]) -> list[str]: ...
    def append_config(self, args: Sequence[str] | None, filename: str | None, *, label: str | None = None) -> None: ...
    @property
    def all_args(self) -> Iterator[str]: ...
    def parse_known_args(self, *, values: optparse.Values | None = None, strict: bool = True) -> tuple[Values, list[str]]: ...
    def parse_args(self) -> tuple[Values, list[str]]: ...

def merge_headers(*dicts: dict[str, Any]) -> dict[str, Any]: ...
def cached_method(f: Callable[..., Any]) -> Callable[..., Any]: ...

class function_with_repr(Generic[_T]):
    def __init__(self, func: Callable[..., _T], repr_: str | None = None) -> None: ...
    def __call__(self, *args: Any, **kwargs: Any) -> _T: ...  # Arbitrary arguments.
    @classmethod
    def set_repr(cls, repr_: str) -> Callable[..., Any]: ...

class Namespace(types.SimpleNamespace):
    def __iter__(self) -> Iterator[Any]: ...
    @property
    def items_(self) -> dict[str, Any]: ...

MEDIA_EXTENSIONS: Final[Namespace]
KNOWN_EXTENSIONS: Final[tuple[str, ...]]

class _UnsafeExtensionError(Exception):
    ALLOWED_EXTENSIONS: frozenset[str]
    extension: str
    def __init__(self, extension: str, /) -> None: ...
    @classmethod
    def sanitize_extension(cls, extension: str, /, *, prepend: bool = False) -> str: ...

class RetryManager:
    attempt: int
    retries: int
    error_callback: Callable[[BaseException, int, int], Any]
    def __init__(
        self, _retries: int | None, _error_callback: Callable[..., Any], **kwargs: Any  # kwargs passed to _error_callback.
    ) -> None: ...
    @property
    def error(self) -> None: ...
    @error.setter
    def error(self, value: type[NO_DEFAULT] | BaseException) -> None: ...
    def __iter__(self) -> Self: ...
    @staticmethod
    def report_retry(
        e: BaseException,
        count: int,
        retries: int,
        *,
        sleep_func: Callable[..., float | None],
        info: Callable[[str], Any],
        warn: Callable[[str], Any],
        error: Callable[[str], Any] | None = None,
        suffix: str | None = None,
    ) -> None: ...

def make_archive_id(ie: InfoExtractor, video_id: str) -> str: ...
def truncate_string(s: str, left: int, right: int = 0) -> str: ...
def orderedSet_from_options(
    options: Sequence[str], alias_dict: dict[str, Sequence[str]], *, use_regex: bool = False, start: Iterable[Any] | None = None
) -> Iterator[Any]: ...

class FormatSorter:
    regex: str
    default: tuple[str, ...]
    ytdl_default: tuple[str, ...]
    settings: dict[str, Any]
    ydl: YoutubeDL
    def __init__(self, ydl: YoutubeDL, field_preference: _Params) -> None: ...
    def evaluate_params(self, params: _Params, sort_extractor: Collection[str]) -> None: ...
    def print_verbose_info(self, write_debug: Callable[..., None]) -> None: ...
    def calculate_preference(self, format: dict[str, Any]) -> tuple[int, ...]: ...

@overload
def filesize_from_tbr(tbr: None, duration: None) -> None: ...
@overload
def filesize_from_tbr(tbr: int, duration: None) -> None: ...
@overload
def filesize_from_tbr(tbr: None, duration: int) -> None: ...
@overload
def filesize_from_tbr(tbr: int | None, duration: int | None) -> int | None: ...

class _YDLLogger:
    def __init__(self, ydl: YoutubeDL | None = None) -> None: ...
    def debug(self, message: str) -> None: ...
    def info(self, message: str) -> None: ...
    def warning(self, message: str, *, once: bool = False) -> None: ...
    def error(self, message: str, *, is_error: bool = True) -> None: ...
    def stdout(self, message: str) -> None: ...
    def stderr(self, message: str) -> None: ...

class _ProgressState(enum.Enum):
    HIDDEN = 0
    INDETERMINATE = 3
    VISIBLE = 1
    WARNING = 4
    ERROR = 2
    @classmethod
    def from_dict(cls, s: dict[str, Any], /) -> _ProgressState: ...
    def get_ansi_escape(self, /, percent: int | None = None) -> str: ...

if sys.platform == "win32":
    _ENV: TypeAlias = Mapping[str, str]
else:
    _ENV: TypeAlias = Mapping[bytes, StrOrBytesPath] | Mapping[str, StrOrBytesPath]

class Popen(subprocess.Popen[AnyStr]):
    def __init__(
        self,
        args: StrOrBytesPath | Sequence[StrOrBytesPath],
        *remaining: Any,  # Passed to subprocess.Popen.__init__().
        env: _ENV | None = None,
        text: bool = False,
        shell: bool = False,
        **kwargs: Any,  # Passed to subprocess.Popen.__init__().
    ) -> None: ...
    def communicate_or_kill(self, input: AnyStr | None = None, timeout: float | None = None) -> tuple[AnyStr, AnyStr]: ...
    def kill(self, *, timeout: int = 0) -> None: ...
    # kwargs passed to cls.__init__().
    @classmethod
    def run(cls, *args: Any, timeout: int | None = None, **kwargs: Any) -> tuple[AnyStr, AnyStr]: ...

class classproperty:
    # args passed to func().
    def __new__(cls, func: Callable[..., Any] | None = None, *args: Any, cache: bool = False) -> Self: ...
    def __init__(  # pyright: ignore[reportInconsistentConstructor]
        self, func: Callable[..., Any], *, cache: bool = False
    ) -> None: ...
    def __get__(self, _: Unused, cls: type[Any]) -> Any: ...



import io
import os
from _typeshed import Incomplete, StrPath
from typing import AnyStr
from typing_extensions import TypeAlias

NativeIO: TypeAlias = io.StringIO

class Verifier:
    ffi: Incomplete
    preamble: Incomplete
    flags: int | None
    kwds: dict[str, list[str] | tuple[str]]
    tmpdir: StrPath
    sourcefilename: str
    modulefilename: str
    ext_package: str | None
    def __init__(
        self,
        ffi,
        preamble,
        tmpdir: StrPath | None = None,
        modulename: str | None = None,
        ext_package: str | None = None,
        tag: str = "",
        force_generic_engine: bool = False,
        source_extension: str = ".c",
        flags: int | None = None,
        relative_to: os.PathLike[AnyStr] | None = None,
        **kwds: list[str] | tuple[str],
    ) -> None: ...
    def write_source(self, file=None) -> None: ...
    def compile_module(self) -> None: ...
    def load_library(self): ...
    def get_module_name(self) -> str: ...
    def get_extension(self): ...
    def generates_python_module(self) -> bool: ...
    def make_relative_to(
        self, kwds: dict[str, list[str] | tuple[str]], relative_to: os.PathLike[AnyStr] | None
    ) -> dict[str, list[str] | tuple[str]]: ...

def set_tmpdir(dirname: StrPath) -> None: ...
def cleanup_tmpdir(tmpdir: StrPath | None = None, keep_so: bool = False) -> None: ...



from _typeshed import Unused
from abc import abstractmethod
from collections import OrderedDict

from pyasn1.type.tag import TagSet

__all__ = ["encode"]

class AbstractItemEncoder:
    @abstractmethod
    def encode(self, value, encodeFun, **options) -> None: ...

class BooleanEncoder(AbstractItemEncoder):
    def encode(self, value, encodeFun, **options): ...

class IntegerEncoder(AbstractItemEncoder):
    def encode(self, value, encodeFun, **options): ...

class BitStringEncoder(AbstractItemEncoder):
    def encode(self, value, encodeFun, **options): ...

class OctetStringEncoder(AbstractItemEncoder):
    def encode(self, value, encodeFun, **options): ...

class TextStringEncoder(AbstractItemEncoder):
    def encode(self, value, encodeFun, **options): ...

class NullEncoder(AbstractItemEncoder):
    def encode(self, value, encodeFun, **options) -> None: ...

class ObjectIdentifierEncoder(AbstractItemEncoder):
    def encode(self, value, encodeFun, **options): ...

class RealEncoder(AbstractItemEncoder):
    def encode(self, value, encodeFun, **options): ...

class SetEncoder(AbstractItemEncoder):
    protoDict = dict
    def encode(self, value, encodeFun, **options): ...

class SequenceEncoder(SetEncoder):
    protoDict = OrderedDict

class SequenceOfEncoder(AbstractItemEncoder):
    def encode(self, value, encodeFun, **options): ...

class ChoiceEncoder(SequenceEncoder): ...

class AnyEncoder(AbstractItemEncoder):
    def encode(self, value, encodeFun, **options): ...

TAG_MAP: dict[TagSet, AbstractItemEncoder]
TYPE_MAP: dict[int, AbstractItemEncoder]
# deprecated aliases
tagMap = TAG_MAP
typeMap = TYPE_MAP

class SingleItemEncoder:
    TAG_MAP: dict[TagSet, AbstractItemEncoder]
    TYPE_MAP: dict[int, AbstractItemEncoder]

    def __init__(self, tagMap=..., typeMap=..., **ignored: Unused) -> None: ...
    def __call__(self, value, **options): ...

class Encoder:
    SINGLE_ITEM_ENCODER: type[SingleItemEncoder]

    def __init__(self, *, tagMap=..., typeMap=..., **options: Unused): ...
    def __call__(self, pyObject, asn1Spec=None, **options): ...

encode: SingleItemEncoder



from _typeshed import Incomplete

from authlib.oauth2 import OAuth2Error

class AssertionClient:
    DEFAULT_GRANT_TYPE: Incomplete
    ASSERTION_METHODS: Incomplete
    token_auth_class: Incomplete
    oauth_error_class = OAuth2Error
    session: Incomplete
    token_endpoint: Incomplete
    grant_type: Incomplete
    issuer: Incomplete
    subject: Incomplete
    audience: Incomplete
    claims: Incomplete
    scope: Incomplete
    token_auth: Incomplete
    leeway: Incomplete
    def __init__(
        self,
        session,
        token_endpoint,
        issuer,
        subject,
        audience=None,
        grant_type=None,
        claims=None,
        token_placement: str = "header",
        scope=None,
        leeway: int = 60,
        **kwargs,
    ) -> None: ...
    @property
    def token(self): ...
    @token.setter
    def token(self, token) -> None: ...
    def refresh_token(self): ...
    def parse_response_token(self, resp): ...
    def __del__(self) -> None: ...



import requests

class SigV4Auth:
    access_key: str
    secret_key: str
    session_token: str | None
    region: str
    def __init__(self, access_key: str, secret_key: str, session_token: str | None = None, region: str = "us-east-1") -> None: ...
    def add_auth(self, request: requests.PreparedRequest) -> None: ...

def generate_sigv4_auth_request(header_value: str | None = None): ...



from _typeshed import Incomplete, ReadableBuffer
from logging import Logger
from typing import IO, Any, ClassVar
from typing_extensions import Self, TypeAlias

Dataset: TypeAlias = Incomplete  # tablib.Dataset

logger: Logger

class Format:
    def get_title(self) -> type[Self]: ...
    def create_dataset(self, in_stream: str | bytes | IO[Any]) -> Dataset: ...
    def export_data(self, dataset: Dataset, **kwargs: Any) -> Any: ...
    def is_binary(self) -> bool: ...
    def get_read_mode(self) -> str: ...
    def get_extension(self) -> str: ...
    def get_content_type(self) -> str: ...
    @classmethod
    def is_available(cls) -> bool: ...
    def can_import(self) -> bool: ...
    def can_export(self) -> bool: ...

class TablibFormat(Format):
    TABLIB_MODULE: ClassVar[str]
    CONTENT_TYPE: ClassVar[str]
    encoding: str | None
    def __init__(self, encoding: str | None = None) -> None: ...
    def get_format(self) -> type[Any]: ...
    def get_title(self) -> str: ...  # type: ignore[override]
    def create_dataset(self, in_stream: str | bytes | IO[Any], **kwargs: Any) -> Dataset: ...  # type: ignore[override]

class TextFormat(TablibFormat): ...

class CSV(TextFormat):
    def export_data(self, dataset: Dataset, **kwargs: Any) -> str: ...

class JSON(TextFormat):
    def export_data(self, dataset: Dataset, **kwargs: Any) -> str: ...

class YAML(TextFormat):
    def export_data(self, dataset: Dataset, **kwargs: Any) -> str: ...

class TSV(TextFormat):
    def export_data(self, dataset: Dataset, **kwargs: Any) -> str: ...

class ODS(TextFormat):
    def export_data(self, dataset: Dataset, **kwargs: Any) -> bytes: ...

class HTML(TextFormat): ...

class XLS(TablibFormat):
    def export_data(self, dataset: Dataset, **kwargs: Any) -> bytes: ...
    def create_dataset(self, in_stream: bytes) -> Dataset: ...  # type: ignore[override]

class XLSX(TablibFormat):
    def export_data(self, dataset: Dataset, **kwargs: Any) -> bytes: ...
    def create_dataset(self, in_stream: ReadableBuffer) -> Dataset: ...  # type: ignore[override]

DEFAULT_FORMATS: list[type[Format]]
BINARY_FORMATS: list[type[Format]]



import ast
import io
import re
from _typeshed import Incomplete, Unused
from collections.abc import Callable, Iterable
from datetime import datetime
from threading import local as _localbase
from types import CodeType, FunctionType, TracebackType
from typing import Any, NoReturn, overload

localbase = _localbase

class PonyDeprecationWarning(DeprecationWarning): ...

def deprecated(stacklevel: int, message: str) -> None: ...
def decorator(caller, func=None): ...
def decorator_with_params(dec): ...
def cut_traceback(func): ...

cut_traceback_depth: int

@overload
def reraise(exc_type: Unused, exc: None, tb: TracebackType | None) -> None: ...
@overload
def reraise(exc_type: Unused, exc: BaseException, tb: TracebackType | None) -> NoReturn: ...
def throw(exc_type: Exception | Callable[..., Exception], *args, **kwargs) -> NoReturn: ...
def truncate_repr(s: object, max_len: int = 100) -> str: ...

codeobjects: dict[int, CodeType]

def get_codeobject_id(codeobject: CodeType) -> int: ...

lambda_args_cache: dict[int | ast.Lambda, list[str]]

def get_lambda_args(func: FunctionType | ast.Lambda) -> list[str]: ...
def error_method(*args: Unused, **kwargs: Unused) -> NoReturn: ...
def is_ident(string: str) -> bool: ...
def split_name(name: str) -> list[str]: ...
def uppercase_name(name: str) -> str: ...
def lowercase_name(name: str) -> str: ...
def camelcase_name(name: str) -> str: ...
def mixedcase_name(name: str) -> str: ...
def import_module(name: str): ...
def is_absolute_path(filename: str) -> bool: ...
def absolutize_path(filename: str, frame_depth: int) -> str: ...
def current_timestamp() -> str: ...
def datetime2timestamp(d: datetime) -> str: ...
def timestamp2datetime(t: str) -> datetime: ...

expr1_re: re.Pattern[str]
expr2_re: re.Pattern[str]
expr3_re: re.Pattern[str]

def parse_expr(s: str, pos: int = 0) -> tuple[str, bool]: ...
def tostring(x): ...
def strjoin(
    sep: str, strings: Iterable[str], source_encoding: str = "ascii", dest_encoding: str | None = None
) -> str | bytes: ...
def count(*args, **kwargs): ...
def avg(iter: Iterable[float | None]) -> float | None: ...
@overload
def group_concat(items: None, sep: str = ",") -> None: ...
@overload
def group_concat(items: Iterable[object], sep: str = ",") -> str: ...
def coalesce(*args: Any) -> Any: ...
def distinct(iter): ...
def concat(*args) -> str: ...
def between(x: float, a: float, b: float) -> bool: ...
def is_utf8(encoding: str) -> bool: ...
def pickle_ast(val): ...
def unpickle_ast(pickled: io.BytesIO): ...
def copy_ast(tree): ...

class HashableDict(dict[Incomplete, Incomplete]):
    def __hash__(self) -> int: ...  # type: ignore[override]
    def __deepcopy__(self, memo): ...
    __setitem__: Incomplete
    __delitem__: Incomplete
    clear: Incomplete
    pop: Incomplete
    popitem: Incomplete
    setdefault: Incomplete
    update: Incomplete

def deref_proxy(value): ...
def deduplicate(value, deduplication_cache): ...



from collections.abc import Iterator
from typing import ClassVar, TypedDict, type_check_only

from jmespath.exceptions import EmptyExpressionError as EmptyExpressionError, LexerError as LexerError

@type_check_only
class _LexerTokenizeResult(TypedDict):
    type: str
    value: str
    start: int
    end: int

class Lexer:
    START_IDENTIFIER: ClassVar[set[str]]
    VALID_IDENTIFIER: ClassVar[set[str]]
    VALID_NUMBER: ClassVar[set[str]]
    WHITESPACE: ClassVar[set[str]]
    SIMPLE_TOKENS: ClassVar[dict[str, str]]
    def tokenize(self, expression: str) -> Iterator[_LexerTokenizeResult]: ...



from .containers import (
    ContainerConfig as ContainerConfig,
    DeviceRequest as DeviceRequest,
    HostConfig as HostConfig,
    LogConfig as LogConfig,
    Ulimit as Ulimit,
)
from .daemon import CancellableStream as CancellableStream
from .healthcheck import Healthcheck as Healthcheck
from .networks import (
    EndpointConfig as EndpointConfig,
    IPAMConfig as IPAMConfig,
    IPAMPool as IPAMPool,
    NetworkingConfig as NetworkingConfig,
)
from .services import (
    ConfigReference as ConfigReference,
    ContainerSpec as ContainerSpec,
    DNSConfig as DNSConfig,
    DriverConfig as DriverConfig,
    EndpointSpec as EndpointSpec,
    Mount as Mount,
    NetworkAttachmentConfig as NetworkAttachmentConfig,
    Placement as Placement,
    PlacementPreference as PlacementPreference,
    Privileges as Privileges,
    Resources as Resources,
    RestartPolicy as RestartPolicy,
    RollbackConfig as RollbackConfig,
    SecretReference as SecretReference,
    ServiceMode as ServiceMode,
    TaskTemplate as TaskTemplate,
    UpdateConfig as UpdateConfig,
)
from .swarm import SwarmExternalCA as SwarmExternalCA, SwarmSpec as SwarmSpec



from __future__ import annotations

from typing import overload
from typing_extensions import assert_type


class CustomIndex:
    def __index__(self) -> int:
        return 1


# float:

assert_type(round(5.5), int)
assert_type(round(5.5, None), int)
assert_type(round(5.5, 0), float)
assert_type(round(5.5, 1), float)
assert_type(round(5.5, 5), float)
assert_type(round(5.5, CustomIndex()), float)

# int:

assert_type(round(1), int)
assert_type(round(1, 1), int)
assert_type(round(1, None), int)
assert_type(round(1, CustomIndex()), int)

# Protocols:


class WithCustomRound1:
    def __round__(self) -> str:
        return "a"


assert_type(round(WithCustomRound1()), str)
assert_type(round(WithCustomRound1(), None), str)
# Errors:
round(WithCustomRound1(), 1)  # type: ignore
round(WithCustomRound1(), CustomIndex())  # type: ignore


class WithCustomRound2:
    def __round__(self, digits: int) -> str:
        return "a"


assert_type(round(WithCustomRound2(), 1), str)
assert_type(round(WithCustomRound2(), CustomIndex()), str)
# Errors:
round(WithCustomRound2(), None)  # type: ignore
round(WithCustomRound2())  # type: ignore


class WithOverloadedRound:
    @overload
    def __round__(self, ndigits: None = ...) -> str: ...

    @overload
    def __round__(self, ndigits: int) -> bytes: ...

    def __round__(self, ndigits: int | None = None) -> str | bytes:
        return b"" if ndigits is None else ""


assert_type(round(WithOverloadedRound()), str)
assert_type(round(WithOverloadedRound(), None), str)
assert_type(round(WithOverloadedRound(), 1), bytes)



from _typeshed import StrOrBytesPath
from collections.abc import Iterable

from ..zoneinfo import _MetadataType

def rebuild(
    filename: StrOrBytesPath, tag=None, format: str = "gz", zonegroups: Iterable[str] = [], metadata: _MetadataType | None = None
) -> None: ...



from _typeshed import Incomplete

from networkx.classes.graph import Graph, _Node
from networkx.utils.backends import _dispatchable

__all__ = ["min_weighted_dominating_set", "min_edge_dominating_set"]

@_dispatchable
def min_weighted_dominating_set(G: Graph[_Node], weight: str | None = None) -> set[Incomplete]: ...
@_dispatchable
def min_edge_dominating_set(G: Graph[_Node]) -> set[Incomplete]: ...



from _typeshed import FileDescriptorLike
from collections.abc import Mapping
from selectors import BaseSelector, SelectorKey
from typing import Any
from typing_extensions import TypeAlias

from gevent._util import Lazy
from gevent.hub import Hub

__all__ = ["DefaultSelector", "GeventSelector"]

_EventMask: TypeAlias = int

# technically this derives from _BaseSelectorImpl, which does not have type annotations
# but in terms of type checking the only difference is, that we need to add get_map since
# GeventSelector does not override it
class GeventSelector(BaseSelector):
    def __init__(self, hub: Hub | None = None) -> None: ...
    @Lazy
    def hub(self) -> Hub: ...
    def register(self, fileobj: FileDescriptorLike, events: _EventMask, data: Any = None) -> SelectorKey: ...
    def unregister(self, fileobj: FileDescriptorLike) -> SelectorKey: ...
    def select(self, timeout: float | None = None) -> list[tuple[SelectorKey, _EventMask]]: ...
    def close(self) -> None: ...
    def get_map(self) -> Mapping[FileDescriptorLike, SelectorKey]: ...

DefaultSelector = GeventSelector



class PluginApiMixin:
    def configure_plugin(self, name, options): ...
    def create_plugin(self, name, plugin_data_dir, gzip: bool = False): ...
    def disable_plugin(self, name, force: bool = False): ...
    def enable_plugin(self, name, timeout: int = 0): ...
    def inspect_plugin(self, name): ...
    def pull_plugin(self, remote, privileges, name=None): ...
    def plugins(self): ...
    def plugin_privileges(self, name): ...
    def push_plugin(self, name): ...
    def remove_plugin(self, name, force: bool = False): ...
    def upgrade_plugin(self, name, remote, privileges): ...



from ..utils import YoutubeDLError
from .common import RequestHandler, Response

class RequestError(YoutubeDLError):
    handler: RequestHandler | None
    cause: Exception | str | None
    def __init__(
        self, msg: str | None = None, cause: Exception | str | None = None, handler: RequestHandler | None = None
    ) -> None: ...

class UnsupportedRequest(RequestError): ...

class NoSupportingHandlers(RequestError):
    unsupported_errors: list[UnsupportedRequest]
    unexpected_errors: list[UnsupportedRequest]
    def __init__(self, unsupported_errors: list[UnsupportedRequest], unexpected_errors: list[Exception]) -> None: ...

class TransportError(RequestError): ...

class HTTPError(RequestError):
    response: Response
    status: int
    reason: str | None
    redirect_loop: bool

    def __init__(self, response: Response, redirect_loop: bool = False) -> None: ...
    def close(self) -> None: ...

class IncompleteRead(TransportError):
    def __init__(
        self,
        partial: int,
        expected: int | None = None,
        *,
        cause: Exception | str | None = None,
        handler: RequestHandler | None = None,
    ) -> None: ...

class SSLError(TransportError): ...
class CertificateVerifyError(SSLError): ...
class ProxyError(TransportError): ...

network_exceptions: tuple[type[RequestError], ...]



############################################################################
# Copyright(c) Open Law Library. All rights reserved.                      #
# See ThirdPartyNotices.txt in the project root for additional notices.    #
#                                                                          #
# Licensed under the Apache License, Version 2.0 (the "License")           #
# you may not use this file except in compliance with the License.         #
# You may obtain a copy of the License at                                  #
#                                                                          #
#     http: // www.apache.org/licenses/LICENSE-2.0                         #
#                                                                          #
# Unless required by applicable law or agreed to in writing, software      #
# distributed under the License is distributed on an "AS IS" BASIS,        #
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. #
# See the License for the specific language governing permissions and      #
# limitations under the License.                                           #
############################################################################

# Dynamically assigned attributes
ATTR_EXECUTE_IN_THREAD = "execute_in_thread"
ATTR_COMMAND_TYPE = "command"
ATTR_FEATURE_TYPE = "feature"
ATTR_REGISTERED_NAME = "reg_name"
ATTR_REGISTERED_TYPE = "reg_type"

# Parameters
PARAM_LS = "ls"



import re
from datetime import datetime
from typing import Any, Final

TIME_MATCHER: Final[re.Pattern[str]]
MS_SEARCHER: Final[re.Pattern[str]]

def patch_strptime() -> Any: ...
def strptime(date_string, format) -> datetime: ...



from _typeshed import Incomplete

__all__ = ["MinHeap", "PairingHeap", "BinaryHeap"]

class MinHeap:
    class _Item:
        __slots__ = ("key", "value")
        key: Incomplete
        value: Incomplete
        def __init__(self, key, value) -> None: ...

    def __init__(self) -> None: ...
    def min(self) -> tuple[Incomplete, Incomplete]: ...
    def pop(self) -> tuple[Incomplete, Incomplete]: ...
    def get(self, key, default=None): ...
    def insert(self, key, value, allow_increase: bool = False) -> bool: ...
    def __nonzero__(self): ...
    def __bool__(self) -> bool: ...
    def __len__(self) -> int: ...
    def __contains__(self, key) -> bool: ...

class PairingHeap(MinHeap):
    class _Node(MinHeap._Item):
        __slots__ = ("left", "next", "prev", "parent")
        left: Incomplete
        next: Incomplete
        prev: Incomplete
        parent: Incomplete
        def __init__(self, key, value) -> None: ...

    def __init__(self) -> None: ...
    def min(self) -> tuple[Incomplete, Incomplete]: ...
    def pop(self) -> tuple[Incomplete, Incomplete]: ...
    def get(self, key, default=None): ...
    def insert(self, key, value, allow_increase: bool = False) -> bool: ...

class BinaryHeap(MinHeap):
    def __init__(self) -> None: ...
    def min(self) -> tuple[Incomplete, Incomplete]: ...
    def pop(self) -> tuple[Incomplete, Incomplete]: ...
    def get(self, key, default=None): ...
    def insert(self, key, value, allow_increase: bool = False) -> bool: ...



from _typeshed import Incomplete
from collections.abc import Callable
from typing import ClassVar

class XRayMiddleware:
    name: ClassVar[str]
    api: ClassVar[int]
    def __init__(self, recorder) -> None: ...
    def apply(self, callback: Callable[..., Incomplete], route) -> Callable[..., Incomplete]: ...
